###########################################################################################################################################################################
# SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD SIH_RD #
###########################################################################################################################################################################

import psycopg2


"""
Cria o schema do sub banco de dados do SIH_RD (AIH Reduzidas) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do sub banco de dados sih_rd
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela das AIH Reduzidas (RD) (tabela principal do sub banco de dados sih_rd)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rdbr("N_AIH"                   VARCHAR(13),
                                                                  "UF_RD"                   VARCHAR(2),
                                                                  "ANO_RD"                  INTEGER,
                                                                  "MES_RD"                  VARCHAR(2),
                                                                  "IDENT_ID"                VARCHAR(2),
                                                                  "UFZI_ID"                 VARCHAR(6),
                                                                  "ANO_CMPT"                INTEGER,
                                                                  "MES_CMPT"                VARCHAR(2),
                                                                  "ESPEC_ID"                VARCHAR(2),
                                                                  "CGC_HOSP"                VARCHAR(14),
                                                                  "CEP"                     VARCHAR(8),
                                                                  "MUNICRES_ID"             VARCHAR(6),
                                                                  "NASC"                    DATE,
                                                                  "SEXO"                    VARCHAR(2),
                                                                  "UTI_MES_TO"              FLOAT,
                                                                  "MARCAUTI_ID"             VARCHAR(2),
                                                                  "UTI_INT_TO"              FLOAT,
                                                                  "DIAR_ACOM"               FLOAT,
                                                                  "QT_DIARIAS"              FLOAT,
                                                                  "PROCSOLIC_ID"            VARCHAR(10),
                                                                  "PROCREA_ID"              VARCHAR(10),
                                                                  "VAL_SH"                  FLOAT,
                                                                  "VAL_SP"                  FLOAT,
                                                                  "VAL_TOT"                 FLOAT,
                                                                  "VAL_UTI"                 FLOAT,
                                                                  "US_TOT"                  FLOAT,
                                                                  "DI_INTER"                DATE,
                                                                  "DT_SAIDA"                DATE,
                                                                  "DIAGPRINC_ID"            VARCHAR(4),
                                                                  "COBRANCA_ID"             VARCHAR(4),
                                                                  "NATUREZA_ID"             VARCHAR(4),
                                                                  "NATJUR_ID"               VARCHAR(4),
                                                                  "GESTAO_ID"               VARCHAR(4),
                                                                  "IND_VDRL"                NUMERIC,
                                                                  "MUNICMOV_ID"             VARCHAR(6),
                                                                  "IDADE"                   FLOAT,
                                                                  "DIAS_PERM"               FLOAT,
                                                                  "MORTE"                   NUMERIC,
                                                                  "NACIONAL_ID"             VARCHAR(3),
                                                                  "CARINT_ID"               VARCHAR(2),
                                                                  "HOMONIMO"                VARCHAR(2),
                                                                  "NUM_FILHOS"              FLOAT,
                                                                  "INSTRU_ID"               VARCHAR(2),
                                                                  "CID_NOTIF"               VARCHAR(4),
                                                                  "CONTRACEP1_ID"           VARCHAR(2),
                                                                  "CONTRACEP2_ID"           VARCHAR(2),
                                                                  "GESTRISCO"               NUMERIC,
                                                                  "INSC_PN"                 VARCHAR(12),
                                                                  "CBOR_ID"                 VARCHAR(6),
                                                                  "CNAER_ID"                VARCHAR(3),
                                                                  "VINCPREV_ID"             VARCHAR(2),
                                                                  "GESTOR_TP"               VARCHAR(2),
                                                                  "GESTOR_CPF"              VARCHAR(11),
                                                                  "CNES_ID"                 VARCHAR(7),
                                                                  "CNPJ_MANT"               VARCHAR(14),
                                                                  "INFEHOSP"                NUMERIC,
                                                                  "CID_ASSO"                VARCHAR(4),
                                                                  "CID_MORTE"               VARCHAR(4),
                                                                  "COMPLEX_ID"              VARCHAR(2),
                                                                  "FINANC_ID"               VARCHAR(2),
                                                                  "FAECTP_ID"               VARCHAR(6),
                                                                  "REGCT_ID"                VARCHAR(4),
                                                                  "RACACOR_ID"              VARCHAR(2),
                                                                  "ETNIA_ID"                VARCHAR(4),
                                                                  "AUD_JUST"                VARCHAR(100),
                                                                  "SIS_JUST"                VARCHAR(100),
                                                                  "VAL_SH_FED"              FLOAT,
                                                                  "VAL_SP_FED"              FLOAT,
                                                                  "VAL_SH_GES"              FLOAT,
                                                                  "VAL_SP_GES"              FLOAT,
                                                                  "GRUPO_ID"                VARCHAR(2),
                                                                  "SUBGRUPO_ID"             VARCHAR(4),
                                                                  "FORMA_ID"                VARCHAR(6));

                       /*Tabela dos tipos de AIH*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ident("ID"                     VARCHAR(2),
                                                                   "TIPO_AIH"               VARCHAR(66));

                       /*Tabela dos municípios gestores*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufzi("ID"                      VARCHAR(6),
                                                                  "MUNNOME"	                VARCHAR(66),
                                                                  "MUNNOMEX"                VARCHAR(66),
                                                                  "MUNCODDV"                VARCHAR(7),
                                                                  "OBSERV"                  VARCHAR(66),
                                                                  "SITUACAO"                VARCHAR(66),
                                                                  "MUNSINP"	                VARCHAR(66),
                                                                  "MUNSIAFI"                VARCHAR(66),
                                                                  "UFCOD_ID"                VARCHAR(2),
                                                                  "AMAZONIA"                VARCHAR(66),
                                                                  "FRONTEIRA"               VARCHAR(66),
                                                                  "CAPITAL"                 VARCHAR(66),
                                                                  "RSAUDE_ID"               VARCHAR(5),
                                                                  "LATITUDE"                FLOAT,
                                                                  "LONGITUDE"               FLOAT,
                                                                  "ALTITUDE"                FLOAT,
                                                                  "AREA"                    FLOAT,
                                                                  "ANOINST"                 VARCHAR(66),
                                                                  "ANOEXT"                  VARCHAR(66),
                                                                  "SUCESSOR"                VARCHAR(66));

                       /*Tabela dos tipos de leito*/
                       CREATE TABLE IF NOT EXISTS {child_db}.espec("ID"                     VARCHAR(2),
                                                                   "LEITO"                  VARCHAR(66));

                       /*Tabela dos municípios de residência do paciente*/
                       CREATE TABLE IF NOT EXISTS {child_db}.municres("ID"                  VARCHAR(6),
                                                                      "MUNNOME"	            VARCHAR(66),
                                                                      "MUNNOMEX"            VARCHAR(66),
                                                                      "MUNCODDV"            VARCHAR(7),
                                                                      "OBSERV"              VARCHAR(66),
                                                                      "SITUACAO"            VARCHAR(66),
                                                                      "MUNSINP"	            VARCHAR(66),
                                                                      "MUNSIAFI"            VARCHAR(66),
                                                                      "UFCOD_ID"            VARCHAR(2),
                                                                      "AMAZONIA"            VARCHAR(66),
                                                                      "FRONTEIRA"           VARCHAR(66),
                                                                      "CAPITAL"             VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"            FLOAT,
                                                                      "LONGITUDE"           FLOAT,
                                                                      "ALTITUDE"            FLOAT,
                                                                      "AREA"                FLOAT,
                                                                      "ANOINST"             VARCHAR(66),
                                                                      "ANOEXT"              VARCHAR(66),
                                                                      "SUCESSOR"            VARCHAR(66));

                       /*Tabela dos tipos de UTI*/
                       CREATE TABLE IF NOT EXISTS {child_db}.marcauti("ID"                  VARCHAR(2),
                                                                      "TIPO_UTI"            VARCHAR(66));

                       /*Tabela dos tipos de procedimento solicitado*/
                       CREATE TABLE IF NOT EXISTS {child_db}.procsolic("ID"                 VARCHAR(10),
                                                                       "PROCEDIMENTO"       VARCHAR(100));

                      /*Tabela dos tipos de procedimento realizado*/
                      CREATE TABLE IF NOT EXISTS {child_db}.procrea("ID"                    VARCHAR(10),
                                                                    "PROCEDIMENTO"          VARCHAR(100));

                      /*Tabela dos tipos de diagnóstico principal*/
                      CREATE TABLE IF NOT EXISTS {child_db}.diagprinc("ID"                  VARCHAR(4),
                                                                      "DIAGNOSTICO"         VARCHAR(66));

                      /*Tabela dos motivos de saída/permanência*/
                      CREATE TABLE IF NOT EXISTS {child_db}.cobranca("ID"                   VARCHAR(2),
                                                                     "MOTIVO"               VARCHAR(66));

                      /*Tabela dos tipos de natureza jurídica dos hospitais*/
                      CREATE TABLE IF NOT EXISTS {child_db}.natureza("ID"                   VARCHAR(2),
                                                                     "NATUREZA"             VARCHAR(66));

                      /*Tabela das naturezas jurídicas de estabelecimentos*/
                      CREATE TABLE IF NOT EXISTS {child_db}.natjur("ID"                     VARCHAR(4),
                                                                   "NATUREZA"               VARCHAR(100));

                      /*Tabela das naturezas jurídicas de estabelecimentos*/
                      CREATE TABLE IF NOT EXISTS {child_db}.gestao("ID"                     VARCHAR(2),
                                                                   "GESTAO"                 VARCHAR(66));

                      /*Tabela dos municípios de localização do estabelecimento*/
                      CREATE TABLE IF NOT EXISTS {child_db}.municmov("ID"                   VARCHAR(6),
                                                                     "MUNNOME"	            VARCHAR(66),
                                                                     "MUNNOMEX"             VARCHAR(66),
                                                                     "MUNCODDV"             VARCHAR(7),
                                                                     "OBSERV"               VARCHAR(66),
                                                                     "SITUACAO"             VARCHAR(66),
                                                                     "MUNSINP"	            VARCHAR(66),
                                                                     "MUNSIAFI"             VARCHAR(66),
                                                                     "UFCOD_ID"             VARCHAR(2),
                                                                     "AMAZONIA"             VARCHAR(66),
                                                                     "FRONTEIRA"            VARCHAR(66),
                                                                     "CAPITAL"              VARCHAR(66),
                                                                     "RSAUDE_ID"            VARCHAR(5),
                                                                     "LATITUDE"             FLOAT,
                                                                     "LONGITUDE"            FLOAT,
                                                                     "ALTITUDE"             FLOAT,
                                                                     "AREA"                 FLOAT,
                                                                     "ANOINST"              VARCHAR(66),
                                                                     "ANOEXT"               VARCHAR(66),
                                                                     "SUCESSOR"             VARCHAR(66));

                       /*Tabela das nacionalidades dos pacientes*/
                       CREATE TABLE IF NOT EXISTS {child_db}.nacional("ID"                  VARCHAR(3),
                                                                      "NACIONALIDADE"       VARCHAR(66));

                       /*Tabela dos motivos de internação*/
                       CREATE TABLE IF NOT EXISTS {child_db}.carint("ID"                    VARCHAR(2),
                                                                    "MOTIVO"                VARCHAR(66));

                       /*Tabela dos níveis de instrução*/
                       CREATE TABLE IF NOT EXISTS {child_db}.instru("ID"                    VARCHAR(2),
                                                                    "NIVEL"                 VARCHAR(66));

                       /*Tabela dos tipos de contraceptivo (primeiro)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.contracep1("ID"                VARCHAR(2),
                                                                        "CONTRACEPTIVO"     VARCHAR(66));

                       /*Tabela dos tipos de contraceptivo (segundo)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.contracep2("ID"                VARCHAR(2),
                                                                        "CONTRACEPTIVO"     VARCHAR(66));

                       /*Tabela das ocupações dos pacientes*/
                       CREATE TABLE IF NOT EXISTS {child_db}.cbor("ID"                      VARCHAR(6),
                                                                  "OCUPACAO"                VARCHAR(66));

                       /*Tabela das atividades econômicas*/
                       CREATE TABLE IF NOT EXISTS {child_db}.cnaer("ID"                     VARCHAR(3),
                                                                   "ATIVIDADE"              VARCHAR(66));

                       /*Tabela dos tipos de vínculo previdenciário do paciente*/
                       CREATE TABLE IF NOT EXISTS {child_db}.vincprev("ID"                  VARCHAR(2),
                                                                      "VINCULO"             VARCHAR(66));

                       /*Tabela dos estabelecimentos de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.cnes("ID"                      VARCHAR(7),
                                                                  "DESCESTAB"               VARCHAR(66));

                       /*Tabela dos níveis de complexidade de atendimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.complex("ID"                   VARCHAR(2),
                                                                     "COMPLEXIDADE"         VARCHAR(66));

                       /*Tabela dos tipos de recursos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.financ("ID"                    VARCHAR(2),
                                                                    "FONTE"                 VARCHAR(66));

                       /*Tabela dos tipos de recursos FAEC*/
                       CREATE TABLE IF NOT EXISTS {child_db}.faectp("ID"                    VARCHAR(6),
                                                                    "SUBFONTE"              VARCHAR(66));

                       /*Tabela dos tipos de regras contratuais*/
                       CREATE TABLE IF NOT EXISTS {child_db}.regct("ID"                     VARCHAR(4),
                                                                   "REGRA"                  VARCHAR(66));

                       /*Tabela dos tipos de raça do paciente*/
                       CREATE TABLE IF NOT EXISTS {child_db}.racacor("ID"                   VARCHAR(2),
                                                                     "RACA"                 VARCHAR(66));

                       /*Tabela dos tipos de etnia indígena*/
                       CREATE TABLE IF NOT EXISTS {child_db}.etnia("ID"                     VARCHAR(4),
                                                                    "INDIGENA"              VARCHAR(66));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                   "ESTADO"                 VARCHAR(66),
                                                                   "SIGLA_UF"               VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                    "REGIAO"                VARCHAR(66));

                       /*Tabela dos grupos de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.grupo("ID"                     VARCHAR(2),
                                                                   "GRUPO"                  VARCHAR(50));

                       /*Tabela dos subgrupos de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.subgrupo("ID"                  VARCHAR(4),
                                                                      "SUBGRUPO"            VARCHAR(100));

                       /*Tabela das formas de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.forma("ID"                     VARCHAR(6),
                                                                   "FORMA"                  VARCHAR(100));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                VARCHAR(15),
                                                                      "DIRETORIO"           VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"   DATE,
                                                                      "DATA_HORA_CARGA"     TIMESTAMP,
                                                                      "QTD_REGISTROS"       INTEGER);
                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



###########################################################################################################################
# SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY SIH_ANY #
###########################################################################################################################

import os
import time
from datetime import datetime

import numpy as np
import pandas as pd
import psycopg2

from transform.prepare_SIH import DataSihMain, DataSihAuxiliary

###########################################################################################################################
#  pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas #
###########################################################################################################################
###########################################################################################################################
#     AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES     #
###########################################################################################################################
# Função que utiliza "pandas.to_sql" para a inserção de dados não principais no banco de dados "child_db"
def insert_into_most_SIH_RD_tables(path, device, child_db):

    label1 = 'append'
    label2 = 'ID'

    # Cria uma instância da classe "DataSihAuxiliary" do módulo "prepare_SIH" do package "data_wrangling"
    data_sih_auxiliary = DataSihAuxiliary(path)

    # Chama métodos da classe "DataSihAuxiliary" do módulo "prepare_SIH" referentes ao sub-banco de dados sih_rd
    df_TABUF = data_sih_auxiliary.get_TABUF_treated()
    df_TABUF.to_sql('ufcod', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_RSAUDE = data_sih_auxiliary.get_RSAUDE_treated()
    df_RSAUDE.to_sql('rsaude', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CADMUN = data_sih_auxiliary.get_CADMUN_treated()
    df_CADMUN.to_sql('ufzi', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_LEITOS = data_sih_auxiliary.get_LEITOS_treated()
    df_LEITOS.to_sql('espec', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_IDENT = data_sih_auxiliary.get_IDENT_treated()
    df_IDENT.to_sql('ident', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "ufzi"
    df_CADMUN.to_sql('municres', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_MARCAUTI = data_sih_auxiliary.get_MARCAUTI_treated()
    df_MARCAUTI.to_sql('marcauti', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_SIGTAP = data_sih_auxiliary.get_TB_SIGTAP_treated()
    df_TB_SIGTAP.to_sql('procsolic', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "procsolic"
    df_TB_SIGTAP.to_sql('procrea', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CID10 = data_sih_auxiliary.get_CID10_treated()
    df_CID10.to_sql('diagprinc', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_SAIDAPERM = data_sih_auxiliary.get_SAIDAPERM_treated()
    df_SAIDAPERM.to_sql('cobranca', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_NATUREZA = data_sih_auxiliary.get_NATUREZA_treated()
    df_NATUREZA.to_sql('natureza', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_natjur = data_sih_auxiliary.get_natjur_treated()
    df_natjur.to_sql('natjur', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_GESTAO = data_sih_auxiliary.get_GESTAO_treated()
    df_GESTAO.to_sql('gestao', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "ufzi"
    df_CADMUN.to_sql('municmov', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_NACION3D = data_sih_auxiliary.get_NACION3D_treated()
    df_NACION3D.to_sql('nacional', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CARATEND = data_sih_auxiliary.get_CARATEND_treated()
    df_CARATEND.to_sql('carint', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_INSTRU = data_sih_auxiliary.get_INSTRU_treated()
    df_INSTRU.to_sql('instru', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CONTRAC = data_sih_auxiliary.get_CONTRAC_treated()
    df_CONTRAC.to_sql('contracep1', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "contracep1"
    df_CONTRAC.to_sql('contracep2', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CBO = data_sih_auxiliary.get_CBO_treated()
    df_CBO.to_sql('cbor', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CNAE = data_sih_auxiliary.get_CNAE_treated()
    df_CNAE.to_sql('cnaer', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_VINCPREV = data_sih_auxiliary.get_VINCPREV_treated()
    df_VINCPREV.to_sql('vincprev', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CNES = data_sih_auxiliary.get_CNES_treated()
    df_CNES.to_sql('cnes', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_COMPLEX2 = data_sih_auxiliary.get_COMPLEX2_treated()
    df_COMPLEX2.to_sql('complex', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_FINANC = data_sih_auxiliary.get_FINANC_treated()
    df_FINANC.to_sql('financ', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_FAECTP = data_sih_auxiliary.get_FAECTP_treated()
    df_FAECTP.to_sql('faectp', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_REGCT = data_sih_auxiliary.get_REGCT_treated()
    df_REGCT.to_sql('regct', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_RACACOR = data_sih_auxiliary.get_RACACOR_treated()
    df_RACACOR.to_sql('racacor', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_etnia = data_sih_auxiliary.get_etnia_treated()
    df_etnia.to_sql('etnia', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_GRUPO = data_sih_auxiliary.get_TB_GRUPO_treated()
    df_TB_GRUPO.to_sql('grupo', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_SUBGR = data_sih_auxiliary.get_TB_SUBGR_treated()
    df_TB_SUBGR.to_sql('subgrupo', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_FORMA = data_sih_auxiliary.get_TB_FORMA_treated()
    df_TB_FORMA.to_sql('forma', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)


###########################################################################################################################
#  pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas pandas #
###########################################################################################################################
###########################################################################################################################
#     AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES * AUXILIARY TABLES     #
###########################################################################################################################
# Função que utiliza "pandas.to_sql" para a inserção de dados não principais no banco de dados "child_db"
def insert_into_most_SIH_SP_tables(path, device, child_db):
    # Inserção dos dados das tabelas não principais no banco de dados
    label1 = 'append'
    label2 = 'ID'

    # Cria uma instância da classe "DataSihAuxiliary" do módulo "prepare_SIH" do package "data_wrangling"
    data_sih_auxiliary = DataSihAuxiliary(path)

    # Chama métodos da classe "DataSihAuxiliary" do módulo "prepare_SIH" referentes ao sub-banco de dados sih_sp
    df_TB_SIGTAP = data_sih_auxiliary.get_TB_SIGTAP_treated()
    df_TB_SIGTAP.to_sql('spprocrea', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TABUF = data_sih_auxiliary.get_TABUF_treated()
    df_TABUF.to_sql('ufcod', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_RSAUDE = data_sih_auxiliary.get_RSAUDE_treated()
    df_RSAUDE.to_sql('rsaude', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CADMUN = data_sih_auxiliary.get_CADMUN_treated()
    df_CADMUN.to_sql('spgestor', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CNES = data_sih_auxiliary.get_CNES_treated()
    df_CNES.to_sql('spcnes', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "spprocrea"
    df_TB_SIGTAP.to_sql('spatoprof', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "spgestor"
    df_CADMUN.to_sql('spmhosp', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "spgestor"
    df_CADMUN.to_sql('spmpac', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_COMPLEX2 = data_sih_auxiliary.get_COMPLEX2_treated()
    df_COMPLEX2.to_sql('spcomplex', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_FINANC = data_sih_auxiliary.get_FINANC_treated()
    df_FINANC.to_sql('spfinanc', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_FAECTP = data_sih_auxiliary.get_FAECTP_treated()
    df_FAECTP.to_sql('spcofaec', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CBO = data_sih_auxiliary.get_CBO_treated()
    df_CBO.to_sql('sppfcbo', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TP_VAL = data_sih_auxiliary.get_TP_VAL_treated()
    df_TP_VAL.to_sql('intpval', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_S_CLASSEN = data_sih_auxiliary.get_S_CLASSEN_treated()
    df_S_CLASSEN.to_sql('servcla', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_CID10 = data_sih_auxiliary.get_CID10_treated()
    df_CID10.to_sql('spcidpri', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    # Mesmo objeto pandas DataFrame da tabela "spcidpri"
    df_CID10.to_sql('spcidsec', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_GRUPO = data_sih_auxiliary.get_TB_GRUPO_treated()
    df_TB_GRUPO.to_sql('grupo', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_SUBGR = data_sih_auxiliary.get_TB_SUBGR_treated()
    df_TB_SUBGR.to_sql('subgrupo', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)

    df_TB_FORMA = data_sih_auxiliary.get_TB_FORMA_treated()
    df_TB_FORMA.to_sql('forma', con=device, schema=child_db, if_exists=label1, index=False, index_label=label2)


###########################################################################################################################
#    copy_expert+pandas copy_expert+pandas copy_expert+pandas copy_expert+pandas copy_expert+pandas copy_expert+pandas    #
###########################################################################################################################
###########################################################################################################################
#    MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE * MAIN TABLE   #
###########################################################################################################################
# Função que utiliza "copy_expert" para a inserção de dados principais e "pandas.to_sql" para a inserção
# dos respectivos metadados no banco de dados "child_db"
def insert_into_main_table_and_arquivos(file_name, directory, date_ftp, device, child_db, connection_data):
    start = time.time()
    counting_rows = pd.read_sql(f'''SELECT COUNT('NOME') FROM {child_db}.arquivos''', con=device)
    qtd_files_pg = counting_rows.iloc[0]['count']
    print(f'A quantidade de arquivos principais de dados do {child_db} já carregada no {connection_data[0]}/PostgreSQL é {qtd_files_pg}.')

    # Tratamento de dados principais do sih_xx
    base = file_name[0:2]
    state = file_name[2:4]
    year = file_name[4:6]
    month = file_name[6:8]
    main_table = base.lower() + 'br'

    print(f'\nIniciando a lida com o arquivo {base}{state}{year}{month}...')
    # Cria uma instância da classe "DataSihMain" do módulo "prepare_SIH" do package "data_wrangling"
    data_sih_main = DataSihMain(base, state, year, month)
    # Chama método da classe "DataSihMain" do módulo "prepare_SIH" referentes ao sub-banco de dados sih_xx
    df = data_sih_main.get_SIHXXaamm_treated()

    # Inserção das colunas UF_XX, ANO_XX, MES_XX e CONTAGEM no objeto pandas DataFrame "df"
    df.insert(1, 'UF_' + base, [state]*df.shape[0])
    df.insert(2, 'ANO_' + base, [int('20' + year)]*df.shape[0])
    df.insert(3, 'MES_' + base, [month]*df.shape[0])

    # Criação de arquivo "csv" contendo os dados do arquivo principal de dados do sih_xx armazenado no...
    # objeto pandas DataFrame "df"
    df.to_csv(base + state + year + month + '.csv', sep=',', header=False, index=False, escapechar=' ')
    # Leitura do arquivo "csv" contendo os dados do arquivo principal de dados do sih_xx
    f = open(base + state + year + month + '.csv', 'r')
    # Conecta ao banco de dados mãe "connection_data[0]" do SGBD PostgreSQL usando o módulo python "psycopg2"
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])
    # Criação de um cursor da conexão tipo "psycopg2" referenciado à variável "cursor"
    cursor = conn.cursor()
    try:
        # Faz a inserção dos dados armazenados em "f" na tabela "main_table" do banco de dados "child_db"...
        # usando o método "copy_expert" do "psycopg2"
        cursor.copy_expert(f'''COPY {child_db}.{main_table} FROM STDIN WITH CSV DELIMITER AS ',';''', f)
    except:
        print(f'Tentando a inserção do arquivo {base}{state}{year}{month} por método alternativo (pandas)...')
        df.to_sql(main_table, con=device, schema=child_db, if_exists='append', index=False)
    else:
        conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()
    # Encerra o file handler
    f.close()
    # Remoção do arquivo "csv"
    os.remove(base + state + year + month + '.csv')
    print(f'Terminou de inserir os dados do arquivo {base}{state}{year}{month} na tabela {main_table} do banco de dados {child_db}.')

    # Cria um objeto pandas DataFrame com apenas uma linha de dados, a qual contém informações sobre o...
    # arquivo de dados principal carregado
    file_data = pd.DataFrame(data=[[file_name, directory, date_ftp, datetime.today(), int(df.shape[0])]],
                             columns= ['NOME', 'DIRETORIO', 'DATA_INSERCAO_FTP', 'DATA_HORA_CARGA', 'QTD_REGISTROS'],
                             index=None
                             )
    # Inserção de informações do arquivo principal de dados no banco de dados "child_db"
    file_data.to_sql('arquivos', con=device, schema=child_db, if_exists='append', index=False)
    print(f'Terminou de inserir os metadados do arquivo {base}{state}{year}{month} na tabela arquivos do banco de dados {child_db}.')
    end = time.time()
    print(f'Demorou {round((end - start), 1)} segundos para essas duas inserções no {connection_data[0]}/PostgreSQL!')



#############################################################################################################################################################
# SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG SINAN_DENG #
#############################################################################################################################################################

import psycopg2


"""
Cria o schema do sub banco de dados do SINAN_DENG (Agravos dengue e chikungunya) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e Microsoft SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do sub banco de dados sinan_deng
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela de Notificações dos Agravos Dengue e Chikungunya (tabela principal do sub banco de dados sinan_deng)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.dengbr("NU_NOTIFIC"            VARCHAR(7),
                                                                    "UF_DENG"               VARCHAR(2),
                                                                    "ANO_DENG"              INTEGER,
                                                                    "TP_NOT"                VARCHAR(2),
                                                                    "ID_AGRAVO"             VARCHAR(3),
                                                                    "MUNICIP_ID"            VARCHAR(6),
                                                                    "RES_CHIKS1"            VARCHAR(2),
                                                                    "RES_CHIKS2"            VARCHAR(2),
                                                                    "RESUL_PRNT"            VARCHAR(2),
                                                                    "RESUL_SORO"            VARCHAR(2),
                                                                    "RESUL_NS1"             VARCHAR(2),
                                                                    "SOROTIPO"              VARCHAR(2),
                                                                    "HOSPITALIZ"            VARCHAR(2),
                                                                    "CLASSIFIN_ID"          VARCHAR(2),
                                                                    "EVOLUCAO"              VARCHAR(2),
                                                                    "GRAV_PULSO"            NUMERIC,
                                                                    "GRAV_CONV"             NUMERIC,
                                                                    "GRAV_ENCH"             NUMERIC,
                                                                    "GRAV_INSUF"            NUMERIC,
                                                                    "GRAV_TAQUI"            NUMERIC,
                                                                    "GRAV_EXTRE"            NUMERIC,
                                                                    "GRAV_HIPOT"            NUMERIC,
                                                                    "GRAV_HEMAT"            NUMERIC,
                                                                    "GRAV_MELEN"            NUMERIC,
                                                                    "GRAV_METRO"            NUMERIC,
                                                                    "GRAV_SANG"             NUMERIC,
                                                                    "GRAV_AST"              NUMERIC,
                                                                    "GRAV_MIOC"             NUMERIC,
                                                                    "GRAV_CONSC"            NUMERIC,
                                                                    "GRAV_ORGAO"            NUMERIC);

                       /*Tabela dos municípios de notificação*/
                       CREATE TABLE IF NOT EXISTS {child_db}.municip("ID"                   VARCHAR(6),
                                                                     "MUNNOME"	            VARCHAR(66),
                                                                     "MUNNOMEX"             VARCHAR(66),
                                                                     "MUNCODDV"             VARCHAR(7),
                                                                     "OBSERV"               VARCHAR(66),
                                                                     "SITUACAO"             VARCHAR(66),
                                                                     "MUNSINP"	            VARCHAR(66),
                                                                     "MUNSIAFI"             VARCHAR(66),
                                                                     "UFCOD_ID"             VARCHAR(2),
                                                                     "AMAZONIA"             VARCHAR(66),
                                                                     "FRONTEIRA"            VARCHAR(66),
                                                                     "CAPITAL"              VARCHAR(66),
                                                                     "RSAUDE_ID"            VARCHAR(5),
                                                                     "LATITUDE"             FLOAT,
                                                                     "LONGITUDE"            FLOAT,
                                                                     "ALTITUDE"             FLOAT,
                                                                     "AREA"                 FLOAT,
                                                                     "ANOINST"              VARCHAR(66),
                                                                     "ANOEXT"               VARCHAR(66),
                                                                     "SUCESSOR"             VARCHAR(66));

                       /*Tabela dos tipos de classificação da dengue/chikungunya*/
                       CREATE TABLE IF NOT EXISTS {child_db}.classifin("ID"                 VARCHAR(2),
                                                                       "CLASSIFICACAO"      VARCHAR(66));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                   "ESTADO"                 VARCHAR(66),
                                                                   "SIGLA_UF"               VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                    "REGIAO"                VARCHAR(66));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                VARCHAR(15),
                                                                      "DIRETORIO"           VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"   DATE,
                                                                      "DATA_HORA_CARGA"     TIMESTAMP,
                                                                      "QTD_REGISTROS"       INTEGER);

                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



########################################################################################################################################################
#  SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC SINASC  #
########################################################################################################################################################

import psycopg2


"""
Cria o schema do banco de dados do SINASC para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e Microsoft SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do banco de dados sinasc
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Declarações de Nascimento (tabela principal do banco de dados sinasc)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.dnbr("NUMERODN"                        VARCHAR(8),
                                                                  "UF_DN"                           VARCHAR(2),
                                                                  "ANO_DN"                          INTEGER,
                                                                  "CODINST"                         VARCHAR(18),
                                                                  "CODESTAB_ID"                     VARCHAR(7),
                                                                  "CODMUNNASC_ID"                   VARCHAR(6),
                                                                  "LOCNASC_ID"                      VARCHAR(2),
                                                                  "IDADEMAE"                        FLOAT,
                                                                  "ESTCIVMAE_ID"                    VARCHAR(2),
                                                                  "ESCMAE_ID"                       VARCHAR(2),
                                                                  "CODOCUPMAE_ID"                   VARCHAR(6),
                                                                  "QTDFILVIVO"                      FLOAT,
                                                                  "QTDFILMORT"                      FLOAT,
                                                                  "CODMUNRES_ID"                    VARCHAR(6),
                                                                  "GESTACAO_ID"                     VARCHAR(2),
                                                                  "GRAVIDEZ_ID"                     VARCHAR(2),
                                                                  "PARTO_ID"                        VARCHAR(2),
                                                                  "CONSULTAS_ID"                    VARCHAR(2),
                                                                  "DTNASC"                          DATE,
                                                                  "HORANASC"                        VARCHAR(4),
                                                                  "SEXO"                            VARCHAR(2),
                                                                  "APGAR1"                          FLOAT,
                                                                  "APGAR5"                          FLOAT,
                                                                  "RACACOR_ID"                      VARCHAR(2),
                                                                  "PESO"                            FLOAT,
                                                                  "IDANOMAL"                        NUMERIC,
                                                                  "DTCADASTRO"                      DATE,
                                                                  "CODANOMAL_ID"                    VARCHAR(4),
                                                                  "DTRECEBIM"                       DATE,
                                                                  "DIFDATA"                         FLOAT,
                                                                  "NATURALMAE_ID"                   VARCHAR(3),
                                                                  "CODMUNNATU_ID"                   VARCHAR(6),
                                                                  "ESCMAE2010_ID"                   VARCHAR(2),
                                                                  "DTNASCMAE"                       DATE,
                                                                  "RACACORMAE_ID"                   VARCHAR(2),
                                                                  "QTDGESTANT"                      FLOAT,
                                                                  "QTDPARTNOR"                      FLOAT,
                                                                  "QTDPARTCES"                      FLOAT,
                                                                  "IDADEPAI"                        FLOAT,
                                                                  "DTULTMENST"                      DATE,
                                                                  "SEMAGESTAC"                      FLOAT,
                                                                  "TPMETESTIM_ID"                   VARCHAR(2),
                                                                  "CONSPRENAT"                      FLOAT,
                                                                  "MESPRENAT"                       VARCHAR(2),
                                                                  "TPAPRESENT_ID"                   VARCHAR(2),
                                                                  "STTRABPART_ID"                   VARCHAR(2),
                                                                  "STCESPARTO_ID"                   VARCHAR(2),
                                                                  "TPNASCASSI_ID"                   VARCHAR(2),
                                                                  "TPFUNCRESP_ID"                   VARCHAR(2),
                                                                  "TPDOCRESP"                       VARCHAR(5),
                                                                  "DTDECLARAC"                      DATE,
                                                                  "ESCMAEAGR1_ID"                   VARCHAR(2),
                                                                  "STDNEPIDEM"                      NUMERIC,
                                                                  "STDNNOVA"                        NUMERIC,
                                                                  "CODPAISRES_ID"                   VARCHAR(3),
                                                                  "TPROBSON_ID"                     VARCHAR(2),
                                                                  "PARIDADE"                        NUMERIC,
                                                                  "KOTELCHUCK"                      VARCHAR(2));

                       /*Tabela dos estabelecimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codestab("ID"                          VARCHAR(7),
                                                                      "DESCESTAB"                   VARCHAR(66),
                                                                      "ESFERA"                      VARCHAR(66),
                                                                      "REGIME"                      VARCHAR(66));

                       /*Tabela dos municípios de nascimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codmunnasc("ID"                        VARCHAR(6),
                                                                        "MUNNOME"	                VARCHAR(66),
                                                                        "MUNNOMEX"                  VARCHAR(66),
                                                                        "MUNCODDV"                  VARCHAR(7),
                                                                        "OBSERV"                    VARCHAR(66),
                                                                        "SITUACAO"                  VARCHAR(66),
                                                                        "MUNSINP"	                VARCHAR(66),
                                                                        "MUNSIAFI"                  VARCHAR(66),
                                                                        "UFCOD_ID"                  VARCHAR(2),
                                                                        "AMAZONIA"                  VARCHAR(66),
                                                                        "FRONTEIRA"                 VARCHAR(66),
                                                                        "CAPITAL"                   VARCHAR(66),
                                                                        "RSAUDE_ID"                 VARCHAR(5),
                                                                        "LATITUDE"                  FLOAT,
                                                                        "LONGITUDE"                 FLOAT,
                                                                        "ALTITUDE"                  FLOAT,
                                                                        "AREA"                      FLOAT,
                                                                        "ANOINST"                   VARCHAR(66),
                                                                        "ANOEXT"                    VARCHAR(66),
                                                                        "SUCESSOR"                  VARCHAR(66));

                       /*Tabela dos locais de nascimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.locnasc("ID"                           VARCHAR(2),
                                                                     "LOCAL"                        VARCHAR(66));

                       /*Tabela de estados civis da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.estcivmae("ID"                         VARCHAR(2),
                                                                       "SITUACAO"                   VARCHAR(66));

                       /*Tabela das faixas de anos de instrução da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.escmae("ID"                            VARCHAR(2),
                                                                    "FAIXA_DE_ANOS_INSTRUCAO"       VARCHAR(66));

                       /*Tabela das ocupações da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codocupmae("ID"                        VARCHAR(6),
                                                                  "OCUPACAO"                        VARCHAR(66));

                       /*Tabela dos municípios de residência da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codmunres("ID"                         VARCHAR(6),
                                                                       "MUNNOME"	                VARCHAR(66),
                                                                       "MUNNOMEX"                   VARCHAR(66),
                                                                       "MUNCODDV"                   VARCHAR(7),
                                                                       "OBSERV"                     VARCHAR(66),
                                                                       "SITUACAO"                   VARCHAR(66),
                                                                       "MUNSINP"	                VARCHAR(66),
                                                                       "MUNSIAFI"                   VARCHAR(66),
                                                                       "UFCOD_ID"                   VARCHAR(2),
                                                                       "AMAZONIA"                   VARCHAR(66),
                                                                       "FRONTEIRA"                  VARCHAR(66),
                                                                       "CAPITAL"                    VARCHAR(66),
                                                                       "RSAUDE_ID"                  VARCHAR(5),
                                                                       "LATITUDE"                   FLOAT,
                                                                       "LONGITUDE"                  FLOAT,
                                                                       "ALTITUDE"                   FLOAT,
                                                                       "AREA"                       FLOAT,
                                                                       "ANOINST"                    VARCHAR(66),
                                                                       "ANOEXT"                     VARCHAR(66),
                                                                       "SUCESSOR"                   VARCHAR(66));

                       /*Tabela das faixas de semanas de gestação*/
                       CREATE TABLE IF NOT EXISTS {child_db}.gestacao("ID"                          VARCHAR(2),
                                                                      "FAIXA_DE_SEMANAS_GESTACAO"   VARCHAR(66));

                       /*Tabela das multiplicidades da gestação*/
                       CREATE TABLE IF NOT EXISTS {child_db}.gravidez("ID"                          VARCHAR(2),
                                                                      "MULTIPLICIDADE_GESTACAO"     VARCHAR(66));

                       /*Tabela dos tipos de parto*/
                       CREATE TABLE IF NOT EXISTS {child_db}.parto("ID"                             VARCHAR(2),
                                                                   "TIPO"                           VARCHAR(66));

                       /*Tabela das faixas do número de consultas*/
                       CREATE TABLE IF NOT EXISTS {child_db}.consultas("ID"                         VARCHAR(2),
                                                                       "FAIXA_DE_NUMERO_CONSULTAS"  VARCHAR(66));

                       /*Tabela dos tipos de raça do RN*/
                       CREATE TABLE IF NOT EXISTS {child_db}.racacor("ID"                           VARCHAR(2),
                                                                     "TIPO"                         VARCHAR(66));

                       /*Tabela dos tipos de anomalia do RN*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codanomal("ID"                         VARCHAR(4),
                                                                       "ANOMALIA"                   VARCHAR(66));

                       /*Tabela do local de nascimento da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.naturalmae("ID"                        VARCHAR(3),
                                                                        "LOCAL"                     VARCHAR(66));

                       /*Tabela dos municípios de nascimento da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codmunnatu("ID"                        VARCHAR(6),
                                                                        "MUNNOME"	                VARCHAR(66),
                                                                        "MUNNOMEX"                  VARCHAR(66),
                                                                        "MUNCODDV"                  VARCHAR(7),
                                                                        "OBSERV"                    VARCHAR(66),
                                                                        "SITUACAO"                  VARCHAR(66),
                                                                        "MUNSINP"	                VARCHAR(66),
                                                                        "MUNSIAFI"                  VARCHAR(66),
                                                                        "UFCOD_ID"                  VARCHAR(2),
                                                                        "AMAZONIA"                  VARCHAR(66),
                                                                        "FRONTEIRA"                 VARCHAR(66),
                                                                        "CAPITAL"                   VARCHAR(66),
                                                                        "RSAUDE_ID"                 VARCHAR(5),
                                                                        "LATITUDE"                  FLOAT,
                                                                        "LONGITUDE"                 FLOAT,
                                                                        "ALTITUDE"                  FLOAT,
                                                                        "AREA"                      FLOAT,
                                                                        "ANOINST"                   VARCHAR(66),
                                                                        "ANOEXT"                    VARCHAR(66),
                                                                        "SUCESSOR"                  VARCHAR(66));

                       /*Tabela das escolaridades da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.escmae2010("ID"                        VARCHAR(2),
                                                                        "ESCOLARIDADE"              VARCHAR(66));

                       /*Tabela dos tipos de raça da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.racacormae("ID"                        VARCHAR(2),
                                                                        "TIPO"                      VARCHAR(66));

                       /*Tabela dos métodos utilizados para saber o número de semanas de gestação*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpmetestim("ID"                        VARCHAR(2),
                                                                        "METODO"                    VARCHAR(66));

                       /*Tabela dos tipos de posição do RN*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpapresent("ID"                        VARCHAR(2),
                                                                        "POSICAO"                   VARCHAR(66));

                       /*Tabela da indução ou não do parto*/
                       CREATE TABLE IF NOT EXISTS {child_db}.sttrabpart("ID"                        VARCHAR(2),
                                                                        "INDUCAO"                   VARCHAR(66));

                       /*Tabela se a cesárea ocorreu ou não antes do parto iniciar*/
                       CREATE TABLE IF NOT EXISTS {child_db}.stcesparto("ID"                        VARCHAR(2),
                                                                        "CESAREA_ANTES_PARTO"       VARCHAR(66));

                       /*Tabela dos tipos de profissional que assistem parto*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpnascassi("ID"                        VARCHAR(2),
                                                                        "ASSISTENCIA"               VARCHAR(66));

                       /*Tabela dos tipos de função do responsável pelo preenchimento da DN*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpfuncresp("ID"                        VARCHAR(2),
                                                                        "FUNCAO"                    VARCHAR(66));

                       /*Tabela das escolaridades da mãe*/
                       CREATE TABLE IF NOT EXISTS {child_db}.escmaeagr1("ID"                        VARCHAR(2),
                                                                        "ESCOLARIDADE"              VARCHAR(66));

                       /*Tabela do país de nascimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codpaisres("ID"                        VARCHAR(3),
                                                                        "PAIS"                      VARCHAR(66));

                       /*Tabela da classificação de Robson*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tprobson("ID"                          VARCHAR(2),
                                                                      "DESCRICAO"                   VARCHAR(66));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                             VARCHAR(2),
                                                                   "ESTADO"                         VARCHAR(66),
                                                                   "SIGLA_UF"                       VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                            VARCHAR(5),
                                                                    "REGIAO"                        VARCHAR(66));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                        VARCHAR(15),
                                                                      "DIRETORIO"                   VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"           DATE,
                                                                      "DATA_HORA_CARGA"             TIMESTAMP,
                                                                      "QTD_REGISTROS"               INTEGER);
                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



###########################################################################################################################################################
# CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST CNES_ST #
###########################################################################################################################################################

import psycopg2


"""
Cria o schema do sub banco de dados do CNES_ST (Estabelecimentos) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e Microsoft SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do sub banco de dados cnes_st
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela Estabelecimentos (tabela principal do sub banco de dados cnes_st)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.stbr("CNES_ID"                 VARCHAR(7),
                                                                  "UF_ST"                   VARCHAR(2),
                                                                  "ANO_ST"                  INTEGER,
                                                                  "MES_ST"                  VARCHAR(2),
                                                                  "CODUFMUN_ID"             VARCHAR(6),
                                                                  "COD_CEP"                 VARCHAR(8),
                                                                  "CPF_CNPJ"                VARCHAR(14),
                                                                  "PFPJ_ID"                 VARCHAR(2),
                                                                  "NIVDEP_ID"               VARCHAR(2),
                                                                  "CNPJ_MAN"                VARCHAR(14),
                                                                  "CODIR_ID"                VARCHAR(2),
                                                                  "VINC_SUS"                NUMERIC,
                                                                  "TPGESTAO_ID"             VARCHAR(2),
                                                                  "ESFERAA_ID"              VARCHAR(2),
                                                                  "RETENCAO_ID"             VARCHAR(2),
                                                                  "ATIVIDAD_ID"             VARCHAR(2),
                                                                  "NATUREZA_ID"             VARCHAR(2),
                                                                  "CLIENTEL_ID"             VARCHAR(2),
                                                                  "TPUNID_ID"               VARCHAR(2),
                                                                  "TURNOAT_ID"              VARCHAR(2),
                                                                  "NIVHIER_ID"              VARCHAR(2),
                                                                  "TPPREST_ID"              VARCHAR(2),
                                                                  "CO_BANCO"                VARCHAR(3),
                                                                  "CO_AGENC"                VARCHAR(5),
                                                                  "C_CORREN"                VARCHAR(14),
                                                                  "ALVARA"                  VARCHAR(25),
                                                                  "DT_EXPED"                DATE,
                                                                  "ORGEXPED_ID"             VARCHAR(2),
                                                                  "AV_ACRED"                NUMERIC,
                                                                  "CLASAVAL_ID"             VARCHAR(2),
                                                                  "DT_ACRED"                DATE,
                                                                  "AV_PNASS"                NUMERIC,
                                                                  "DT_PNASS"                DATE,
                                                                  "GESPRG1E"                NUMERIC,
                                                                  "GESPRG1M"                NUMERIC,
                                                                  "GESPRG2E"                NUMERIC,
                                                                  "GESPRG2M"                NUMERIC,
                                                                  "GESPRG4E"                NUMERIC,
                                                                  "GESPRG4M"                NUMERIC,
                                                                  "NIVATE_A"                NUMERIC,
                                                                  "GESPRG3E"                NUMERIC,
                                                                  "GESPRG3M"                NUMERIC,
                                                                  "GESPRG5E"                NUMERIC,
                                                                  "GESPRG5M"                NUMERIC,
                                                                  "GESPRG6E"                NUMERIC,
                                                                  "GESPRG6M"                NUMERIC,
                                                                  "NIVATE_H"                NUMERIC,
                                                                  "QTLEITP1"                FLOAT,
                                                                  "QTLEITP2"                FLOAT,
                                                                  "QTLEITP3"                FLOAT,
                                                                  "LEITHOSP"                NUMERIC,
                                                                  "QTINST01"                FLOAT,
                                                                  "QTINST02"                FLOAT,
                                                                  "QTINST03"                FLOAT,
                                                                  "QTINST04"                FLOAT,
                                                                  "QTINST05"                FLOAT,
                                                                  "QTINST06"                FLOAT,
                                                                  "QTINST07"                FLOAT,
                                                                  "QTINST08"                FLOAT,
                                                                  "QTINST09"                FLOAT,
                                                                  "QTINST10"                FLOAT,
                                                                  "QTINST11"                FLOAT,
                                                                  "QTINST12"                FLOAT,
                                                                  "QTINST13"                FLOAT,
                                                                  "QTINST14"                FLOAT,
                                                                  "URGEMERG"                NUMERIC,
                                                                  "QTINST15"                FLOAT,
                                                                  "QTINST16"                FLOAT,
                                                                  "QTINST17"                FLOAT,
                                                                  "QTINST18"                FLOAT,
                                                                  "QTINST19"                FLOAT,
                                                                  "QTINST20"                FLOAT,
                                                                  "QTINST21"                FLOAT,
                                                                  "QTINST22"                FLOAT,
                                                                  "QTINST23"                FLOAT,
                                                                  "QTINST24"                FLOAT,
                                                                  "QTINST25"                FLOAT,
                                                                  "QTINST26"                FLOAT,
                                                                  "QTINST27"                FLOAT,
                                                                  "QTINST28"                FLOAT,
                                                                  "QTINST29"                FLOAT,
                                                                  "QTINST30"                FLOAT,
                                                                  "ATENDAMB"                NUMERIC,
                                                                  "QTINST31"                FLOAT,
                                                                  "QTINST32"                FLOAT,
                                                                  "QTINST33"                FLOAT,
                                                                  "CENTRCIR"                NUMERIC,
                                                                  "QTINST34"                FLOAT,
                                                                  "QTINST35"                FLOAT,
                                                                  "QTINST36"                FLOAT,
                                                                  "QTINST37"                FLOAT,
                                                                  "CENTROBS"                NUMERIC,
                                                                  "QTLEIT05"                FLOAT,
                                                                  "QTLEIT06"                FLOAT,
                                                                  "QTLEIT07"                FLOAT,
                                                                  "QTLEIT09"                FLOAT,
                                                                  "QTLEIT19"                FLOAT,
                                                                  "QTLEIT20"                FLOAT,
                                                                  "QTLEIT21"                FLOAT,
                                                                  "QTLEIT22"                FLOAT,
                                                                  "QTLEIT23"                FLOAT,
                                                                  "QTLEIT32"                FLOAT,
                                                                  "QTLEIT34"                FLOAT,
                                                                  "QTLEIT38"                FLOAT,
                                                                  "QTLEIT39"                FLOAT,
                                                                  "QTLEIT40"                FLOAT,
                                                                  "CENTRNEO"                NUMERIC,
                                                                  "ATENDHOS"                NUMERIC,
                                                                  "SERAP01P"                NUMERIC,
                                                                  "SERAP01T"                NUMERIC,
                                                                  "SERAP02P"                NUMERIC,
                                                                  "SERAP02T"                NUMERIC,
                                                                  "SERAP03P"                NUMERIC,
                                                                  "SERAP03T"                NUMERIC,
                                                                  "SERAP04P"                NUMERIC,
                                                                  "SERAP04T"                NUMERIC,
                                                                  "SERAP05P"                NUMERIC,
                                                                  "SERAP05T"                NUMERIC,
                                                                  "SERAP06P"                NUMERIC,
                                                                  "SERAP06T"                NUMERIC,
                                                                  "SERAP07P"                NUMERIC,
                                                                  "SERAP07T"                NUMERIC,
                                                                  "SERAP08P"                NUMERIC,
                                                                  "SERAP08T"                NUMERIC,
                                                                  "SERAP09P"                NUMERIC,
                                                                  "SERAP09T"                NUMERIC,
                                                                  "SERAP10P"                NUMERIC,
                                                                  "SERAP10T"                NUMERIC,
                                                                  "SERAP11P"                NUMERIC,
                                                                  "SERAP11T"                NUMERIC,
                                                                  "SERAPOIO"                NUMERIC,
                                                                  "RES_BIOL"                NUMERIC,
                                                                  "RES_QUIM"                NUMERIC,
                                                                  "RES_RADI"                NUMERIC,
                                                                  "RES_COMU"                NUMERIC,
                                                                  "COLETRES"                NUMERIC,
                                                                  "COMISS01"                NUMERIC,
                                                                  "COMISS02"                NUMERIC,
                                                                  "COMISS03"                NUMERIC,
                                                                  "COMISS04"                NUMERIC,
                                                                  "COMISS05"                NUMERIC,
                                                                  "COMISS06"                NUMERIC,
                                                                  "COMISS07"                NUMERIC,
                                                                  "COMISS08"                NUMERIC,
                                                                  "COMISS09"                NUMERIC,
                                                                  "COMISS10"                NUMERIC,
                                                                  "COMISS11"                NUMERIC,
                                                                  "COMISS12"                NUMERIC,
                                                                  "COMISSAO"                NUMERIC,
                                                                  "AP01CV01"                NUMERIC,
                                                                  "AP01CV02"                NUMERIC,
                                                                  "AP01CV05"                NUMERIC,
                                                                  "AP01CV06"                NUMERIC,
                                                                  "AP01CV03"                NUMERIC,
                                                                  "AP01CV04"                NUMERIC,
                                                                  "AP02CV01"                NUMERIC,
                                                                  "AP02CV02"                NUMERIC,
                                                                  "AP02CV05"                NUMERIC,
                                                                  "AP02CV06"                NUMERIC,
                                                                  "AP02CV03"                NUMERIC,
                                                                  "AP02CV04"                NUMERIC,
                                                                  "AP03CV01"                NUMERIC,
                                                                  "AP03CV02"                NUMERIC,
                                                                  "AP03CV05"                NUMERIC,
                                                                  "AP03CV06"                NUMERIC,
                                                                  "AP03CV03"                NUMERIC,
                                                                  "AP03CV04"                NUMERIC,
                                                                  "AP04CV01"                NUMERIC,
                                                                  "AP04CV02"                NUMERIC,
                                                                  "AP04CV05"                NUMERIC,
                                                                  "AP04CV06"                NUMERIC,
                                                                  "AP04CV03"                NUMERIC,
                                                                  "AP04CV04"                NUMERIC,
                                                                  "AP05CV01"                NUMERIC,
                                                                  "AP05CV02"                NUMERIC,
                                                                  "AP05CV05"                NUMERIC,
                                                                  "AP05CV06"                NUMERIC,
                                                                  "AP05CV03"                NUMERIC,
                                                                  "AP05CV04"                NUMERIC,
                                                                  "AP06CV01"                NUMERIC,
                                                                  "AP06CV02"                NUMERIC,
                                                                  "AP06CV05"                NUMERIC,
                                                                  "AP06CV06"                NUMERIC,
                                                                  "AP06CV03"                NUMERIC,
                                                                  "AP06CV04"                NUMERIC,
                                                                  "AP07CV01"                NUMERIC,
                                                                  "AP07CV02"                NUMERIC,
                                                                  "AP07CV05"                NUMERIC,
                                                                  "AP07CV06"                NUMERIC,
                                                                  "AP07CV03"                NUMERIC,
                                                                  "AP07CV04"                NUMERIC,
                                                                  "ATEND_PR"                NUMERIC,
                                                                  "NATJUR_ID"               VARCHAR(4));

                       /*Tabela dos estabelecimentos de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.cnes("ID"                      VARCHAR(7),
                                                                  "DESCESTAB"               VARCHAR(66),
                                                                  "RSOC_MAN"                VARCHAR(66),
                                                                  "CPF_CNPJ"                VARCHAR(14),
                                                                  "EXCLUIDO"                NUMERIC,
                                                                  "DATAINCL"                DATE,
                                                                  "DATAEXCL"                DATE);

                       /*Tabela dos municípios de localização de estabelecimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codufmun("ID"                  VARCHAR(6),
                                                                      "MUNNOME"	            VARCHAR(66),
                                                                      "MUNNOMEX"            VARCHAR(66),
                                                                      "MUNCODDV"            VARCHAR(7),
                                                                      "OBSERV"              VARCHAR(66),
                                                                      "SITUACAO"            VARCHAR(66),
                                                                      "MUNSINP"	            VARCHAR(66),
                                                                      "MUNSIAFI"            VARCHAR(66),
                                                                      "UFCOD_ID"            VARCHAR(2),
                                                                      "AMAZONIA"            VARCHAR(66),
                                                                      "FRONTEIRA"           VARCHAR(66),
                                                                      "CAPITAL"             VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"            FLOAT,
                                                                      "LONGITUDE"           FLOAT,
                                                                      "ALTITUDE"            FLOAT,
                                                                      "AREA"                FLOAT,
                                                                      "ANOINST"             VARCHAR(66),
                                                                      "ANOEXT"              VARCHAR(66),
                                                                      "SUCESSOR"            VARCHAR(66));

                       /*Tabela se o estabelecimento é pessoa física ou pessoa jurídica*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pfpj("ID"                      VARCHAR(2),
                                                                  "PESSOA"                  VARCHAR(66));

                       /*Tabela do grau de independência do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.nivdep("ID"                    VARCHAR(2),
                                                                    "TIPO"                  VARCHAR(66));

                       /*Tabela dos tipos de retenção de tributos da mantenedora*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codir("ID"                     VARCHAR(2),
                                                                   "RETENCAO"               VARCHAR(66));

                       /*Tabela dos tipos de gestão*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpgestao("ID"                  VARCHAR(2),
                                                                      "GESTAO"              VARCHAR(66));

                       /*Tabela dos tipos de administração*/
                       CREATE TABLE IF NOT EXISTS {child_db}.esferaa("ID"                   VARCHAR(2),
                                                                     "ADMINISTRACAO"        VARCHAR(66));

                       /*Tabela dos tipos de retenção de tributos do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.retencao("ID"                  VARCHAR(2),
                                                                      "RETENCAO"            VARCHAR(66));

                       /*Tabela dos tipos de atividade de ensino, se houver*/
                       CREATE TABLE IF NOT EXISTS {child_db}.atividad("ID"                  VARCHAR(2),
                                                                      "ATIVIDADE"           VARCHAR(66));

                       /*Tabela da natureza do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.natureza("ID"                  VARCHAR(2),
                                                                      "NATUREZA"            VARCHAR(66));

                       /*Tabela dos tipos de fluxo de clientela*/
                       CREATE TABLE IF NOT EXISTS {child_db}.clientel("ID"                  VARCHAR(2),
                                                                      "CLIENTELA"           VARCHAR(66));

                       /*Tabela dos tipos de estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpunid("ID"                    VARCHAR(2),
                                                                    "TIPO"                  VARCHAR(66));

                       /*Tabela dos turnos de funcionamento do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.turnoat("ID"                   VARCHAR(2),
                                                                     "TURNO"                VARCHAR(66));

                       /*Tabela dos níveis de atendimento do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.nivhier("ID"                   VARCHAR(2),
                                                                     "NIVEL"                VARCHAR(66));

                       /*Tabela dos tipos de prestador dos serviços hospitalares*/
                       CREATE TABLE IF NOT EXISTS {child_db}.tpprest("ID"                   VARCHAR(2),
                                                                     "PRESTADOR"            VARCHAR(66));

                       /*Tabela dos tipos de órgão expedidor de alvará*/
                       CREATE TABLE IF NOT EXISTS {child_db}.orgexped("ID"                  VARCHAR(2),
                                                                      "EXPEDIDOR"           VARCHAR(66));

                       /*Tabela das classificações de avaliacao de estabelecimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.clasaval("ID"                  VARCHAR(2),
                                                                      "AVALIACAO"           VARCHAR(66));

                       /*Tabela das naturezas jurídicas de estabelecimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.natjur("ID"                    VARCHAR(4),
                                                                    "NATUREZA"              VARCHAR(100));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                   "ESTADO"                 VARCHAR(66),
                                                                   "SIGLA_UF"               VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                    "REGIAO"                VARCHAR(66));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                VARCHAR(15),
                                                                      "DIRETORIO"           VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"   DATE,
                                                                      "DATA_HORA_CARGA"     TIMESTAMP,
                                                                      "QTD_REGISTROS"       INTEGER);

                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



###########################################################################################################################################################################
# SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP SIH_SP #
###########################################################################################################################################################################

import psycopg2


"""
Cria o schema do sub banco de dados do SIH_SP (AIH Serviços Profissionais) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do sub banco de dados sih_sp
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela das AIH Serviços Profissionais (SP) (tabela principal do sub banco de dados SIH_SP)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spbr("SP_NAIH"                 VARCHAR(13),
                                                                  "UF_SP"                   VARCHAR(2),
                                                                  "ANO_SP"                  INTEGER,
                                                                  "MES_SP"                  VARCHAR(2),
                                                                  "SPPROCREA_ID"            VARCHAR(10),
                                                                  "SPGESTOR_ID"             VARCHAR(6),
                                                                  "SPCNES_ID"               VARCHAR(7),
                                                                  "SP_DTINTER"              DATE,
                                                                  "SP_DTSAIDA"              DATE,
                                                                  "SP_CPFCGC"               VARCHAR(14),
                                                                  "SPATOPROF_ID"            VARCHAR(10),
                                                                  "SP_QTD_ATO"              FLOAT,
                                                                  "SP_PTSP"                 FLOAT,
                                                                  "SP_VALATO"               FLOAT,
                                                                  "SPMHOSP_ID"              VARCHAR(6),
                                                                  "SPMPAC_ID"               VARCHAR(6),
                                                                  "SP_DES_HOS"              NUMERIC,
                                                                  "SP_DES_PAC"              NUMERIC,
                                                                  "SPCOMPLEX_ID"            VARCHAR(2),
                                                                  "SPFINANC_ID"             VARCHAR(2),
                                                                  "SPCOFAEC_ID"             VARCHAR(6),
                                                                  "SPPFCBO_ID"              VARCHAR(6),
                                                                  "SP_PF_DOC"               VARCHAR(11),
                                                                  "SP_PJ_DOC"               VARCHAR(7),
                                                                  "INTPVAL_ID"              VARCHAR(2),
                                                                  "SERVCLA_ID"              VARCHAR(6),
                                                                  "SPCIDPRI_ID"             VARCHAR(4),
                                                                  "SPCIDSEC_ID"             VARCHAR(4),
                                                                  "SP_QT_PROC"              FLOAT,
                                                                  "SP_U_AIH"                NUMERIC,
                                                                  "GRUPO_ID"                VARCHAR(2),
                                                                  "SUBGRUPO_ID"             VARCHAR(4),
                                                                  "FORMA_ID"                VARCHAR(6));

                       /*Tabela dos tipos de procedimento realizado*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spprocrea("ID"                 VARCHAR(10),
                                                                       "PROCEDIMENTO"       VARCHAR(100));

                       /*Tabela dos municípios gestores*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spgestor("ID"                  VARCHAR(6),
                                                                      "MUNNOME"	            VARCHAR(66),
                                                                      "MUNNOMEX"            VARCHAR(66),
                                                                      "MUNCODDV"            VARCHAR(7),
                                                                      "OBSERV"              VARCHAR(66),
                                                                      "SITUACAO"            VARCHAR(66),
                                                                      "MUNSINP"	            VARCHAR(66),
                                                                      "MUNSIAFI"            VARCHAR(66),
                                                                      "UFCOD_ID"            VARCHAR(2),
                                                                      "AMAZONIA"            VARCHAR(66),
                                                                      "FRONTEIRA"           VARCHAR(66),
                                                                      "CAPITAL"             VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"            FLOAT,
                                                                      "LONGITUDE"           FLOAT,
                                                                      "ALTITUDE"            FLOAT,
                                                                      "AREA"                FLOAT,
                                                                      "ANOINST"             VARCHAR(66),
                                                                      "ANOEXT"              VARCHAR(66),
                                                                      "SUCESSOR"            VARCHAR(66));

                       /*Tabela dos estabelecimentos de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spcnes("ID"                    VARCHAR(7),
                                                                    "DESCESTAB"             VARCHAR(66));

                       /*Tabela dos tipos de procedimento referente a ato profissional*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spatoprof("ID"                 VARCHAR(10),
                                                                       "PROCEDIMENTO"       VARCHAR(100));

                       /*Tabela dos municípios de localização do estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spmhosp("ID"                   VARCHAR(6),
                                                                     "MUNNOME"	            VARCHAR(66),
                                                                     "MUNNOMEX"             VARCHAR(66),
                                                                     "MUNCODDV"             VARCHAR(7),
                                                                     "OBSERV"               VARCHAR(66),
                                                                     "SITUACAO"             VARCHAR(66),
                                                                     "MUNSINP"	            VARCHAR(66),
                                                                     "MUNSIAFI"             VARCHAR(66),
                                                                     "UFCOD_ID"             VARCHAR(2),
                                                                     "AMAZONIA"             VARCHAR(66),
                                                                     "FRONTEIRA"            VARCHAR(66),
                                                                     "CAPITAL"              VARCHAR(66),
                                                                     "RSAUDE_ID"            VARCHAR(5),
                                                                     "LATITUDE"             FLOAT,
                                                                     "LONGITUDE"            FLOAT,
                                                                     "ALTITUDE"             FLOAT,
                                                                     "AREA"                 FLOAT,
                                                                     "ANOINST"              VARCHAR(66),
                                                                     "ANOEXT"               VARCHAR(66),
                                                                     "SUCESSOR"             VARCHAR(66));

                       /*Tabela dos municípios de residência do paciente*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spmpac("ID"                    VARCHAR(6),
                                                                    "MUNNOME"	            VARCHAR(66),
                                                                    "MUNNOMEX"              VARCHAR(66),
                                                                    "MUNCODDV"              VARCHAR(7),
                                                                    "OBSERV"                VARCHAR(66),
                                                                    "SITUACAO"              VARCHAR(66),
                                                                    "MUNSINP"	            VARCHAR(66),
                                                                    "MUNSIAFI"              VARCHAR(66),
                                                                    "UFCOD_ID"              VARCHAR(2),
                                                                    "AMAZONIA"              VARCHAR(66),
                                                                    "FRONTEIRA"             VARCHAR(66),
                                                                    "CAPITAL"               VARCHAR(66),
                                                                    "RSAUDE_ID"             VARCHAR(5),
                                                                    "LATITUDE"              FLOAT,
                                                                    "LONGITUDE"             FLOAT,
                                                                    "ALTITUDE"              FLOAT,
                                                                    "AREA"                  FLOAT,
                                                                    "ANOINST"               VARCHAR(66),
                                                                    "ANOEXT"                VARCHAR(66),
                                                                    "SUCESSOR"              VARCHAR(66));

                       /*Tabela dos níveis de complexidade do ato profissional*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spcomplex("ID"                 VARCHAR(2),
                                                                       "COMPLEXIDADE"       VARCHAR(66));

                       /*Tabela dos tipos de recursos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spfinanc("ID"                  VARCHAR(2),
                                                                      "FONTE"               VARCHAR(66));

                       /*Tabela dos tipos de recursos FAEC*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spcofaec("ID"                  VARCHAR(6),
                                                                      "SUBFONTE"            VARCHAR(66));

                       /*Tabela das ocupações dos profissionais*/
                       CREATE TABLE IF NOT EXISTS {child_db}.sppfcbo("ID"                   VARCHAR(6),
                                                                     "OCUPACAO"             VARCHAR(66));

                       /*Tabela dos tipos de recursos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.intpval("ID"                   VARCHAR(2),
                                                                     "TIPO_VALOR"           VARCHAR(66));

                       /*Tabela de classificações dos serviços*/
                       CREATE TABLE IF NOT EXISTS {child_db}.servcla("ID"                   VARCHAR(6),
                                                                     "CLASSIFICACAO"        VARCHAR(100));

                       /*Tabela dos tipos de diagnóstico principal*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spcidpri("ID"                  VARCHAR(4),
                                                                      "DIAGNOSTICO"         VARCHAR(66));

                       /*Tabela dos tipos de diagnóstico secundário*/
                       CREATE TABLE IF NOT EXISTS {child_db}.spcidsec("ID"                  VARCHAR(4),
                                                                      "DIAGNOSTICO"         VARCHAR(66));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                   "ESTADO"                 VARCHAR(66),
                                                                   "SIGLA_UF"               VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                    "REGIAO"                VARCHAR(66));

                       /*Tabela dos grupos de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.grupo("ID"                     VARCHAR(2),
                                                                   "GRUPO"                  VARCHAR(50));

                       /*Tabela dos subgrupos de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.subgrupo("ID"                  VARCHAR(4),
                                                                      "SUBGRUPO"            VARCHAR(100));

                       /*Tabela das formas de procedimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.forma("ID"                     VARCHAR(6),
                                                                   "FORMA"                  VARCHAR(100));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                VARCHAR(15),
                                                                      "DIRETORIO"           VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"   DATE,
                                                                      "DATA_HORA_CARGA"     TIMESTAMP,
                                                                      "QTD_REGISTROS"       INTEGER);
                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



from datetime import datetime

import numpy as np
import pandas as pd
from ftplib import FTP

"""
Módulo com funções para obtenção de:

1) lista de nomes (e outros metadados) dos arquivos principais de dados de base de dados constante de um
diretório do servidor FTP do Datasus [get_dbc_info];

2) seleciona determinadas linhas da lista de nomes (e outros metadados) dos arquivos principais de dados de
uma base ou uma sub-base de dados do Datasus [files_in_ftp_base/files_in_ftp_subbase];

3) dicionário dos nomes das tabelas auxiliares e o respectivo número de registros constantes de um banco ou
sub-banco de dados PostgreSQL [get_tables_counts_db/get_tables_counts_subdb];

4) dataframe com os nomes (e outros metadados) dos arquivos principais de dados de base de dados do Datasus
já carregados no respectivo banco de dados PostgreSQL [files_loaded];

5) dataframe com os nomes (e outros metadados) dos arquivos principais de dados de base de dados do Datasus
 que ainda faltam inserir no respectivo banco de dados PostgreSQL [files_to_load];
"""


def get_dbc_info(ftp_path):
    """
    Criação de objeto pandas DataFrame contendo nomes (e outros metadados) dos arquivos principais de dados
    de base de dados do Datasus constante do diretório FTP "ftp_path".

    Parâmetros
    ----------
    ftp_path: objeto str
        String do diretório FTP

    Retorno
    -------
    df_files: objeto pandas Dataframe
        Dataframe tendo como colunas o nome, o diretório e a data de inserção de cada arquivo principal de
        um diretório FTP do Datasus
    """

    # Muda para o diretório "ftp_path" onde estão os arquivos principais de dados relativos ao "folder_name"
    ftp.cwd(ftp_path)
    # Coleta os "Nome", "Tamanho" e "Data da modificação" dos arquivos contidos no diretório "ftp_path"...
    # como um objeto string e o coloca no arquivo "stuff_ftp_files.txt"
    ftp.retrlines('LIST', open('stuff_ftp_files.txt', 'w').write)
    # Abre o arquivo "stuff_ftp_files.txt" num objeto file handler referenciado à variável "f"
    with open('stuff_ftp_files.txt') as f:
        # Lê de uma vez o conteúdo do arquivo "stuff_ftp_files.txt" guardando-o num objeto do tipo string...
        # e referenciando-o à variável "string_ftp_files"
        string_ftp_files = f.read()
    # Divide o conteúdo da variável "string_ftp_files" (tornada UPPER CASE) onde haja a substring ".DBC",...
    # colocando cada fatia como um objeto string de um objeto list e desconsiderando o último elemento string...
    # da list por ser uma string vazia já que o objeto string de "string_ftp_files" termina com ".DBC"
    list_ftp_files = string_ftp_files.upper().split('.DBC')[:-1]
    # Concatena ao final de cada elemento string da variável list "list_ftp_files" a string ".dbc"...
    for i in range(len(list_ftp_files)):
        list_ftp_files[i] = list_ftp_files[i] + '.dbc'
    # Cria um objeto list vazio destinado ao nome dos arquivos de extensão "dbc"
    list_file_names = []
    # Cria um objeto list vazio destinado à data de inserção do arquivo "dbc" no diretório ftp
    list_file_dates = []
    # Iterate por cada objeto string do objeto list "list_ftp_files", divide-o nos espaços vazios colocando...
    # cada fatia como um objeto string de um objeto list e pega apenas o primeiro e o último elemento desse...
    # objeto list colocando como um elemento da list "list_file_names"
    for elem in list_ftp_files:
        list_file_dates.append(datetime.strptime(elem.split()[0], '%m-%d-%y').date()) # Já converte o objeto string
                                                                                      # para date
        list_file_names.append(elem.split()[-1])

    # Cria um objeto pandas DataFrame para armazenar o nome dos arquivos de dados "dbc", o diretório onde o...
    # arquivo se encontra e a data de inserção ou reinserção do arquivo no diretório ftp
    df_files = pd.DataFrame(columns=['NOME', 'DIRETORIO', 'DATA_INSERCAO_FTP'])
    # Aloca os dados no objeto pandas DataFrame
    for i in np.arange(len(list_file_names)):
        df_files.loc[i] = [list_file_names[i], ftp_path, list_file_dates[i]]

    return df_files


def files_in_ftp_base(name_base):
    """
    Seleciona determinadas linhas do objeto pandas DataFrame retornado pela função "get_dbc_info" e relativo
    a uma base de dados do Datasus denominada "name_base" em lowercase.

    Parâmetros
    ----------
    name_base: objeto str
        String do nome (lowercase) de uma base de dados do Datasus

    Retorno
    -------
    df_ftp: objeto pandas Dataframe
        Dataframe tendo como colunas o nome, o diretório e a data de inserção dos arquivos principais de dados
        selecionados de uma base de dados do Datasus
    """

    # Host domain
    host = 'ftp.datasus.gov.br'
    # Criação de objeto FTP como uma variável global
    global ftp
    ftp = FTP(host)
    # Realização de conexão
    ftp.login()

    # CNES
    if name_base == 'cnes':
        # Diretório do host onde estão os dados da base de dados "name_base" (CNES) do Datasus
        ftp_path = '/dissemin/publicos/CNES/200508_/Dados/'
        # Muda para o diretório "ftp_path" que contém 13 pastas com os arquivos principais de dados
        ftp.cwd(ftp_path)
        # Obtém o nome dessas 13 pastas do diretório "ftp_path" como um objeto list
        lista_folder_names = ftp.nlst()
        # Cria objeto list vazio
        frames = []
        # Itera sobre cada diretório dos 13 diretórios de dados principais do CNES
        for folder_name in lista_folder_names:
            # Cria o objeto string final do path do diretório onde estão os arquivos principais de dados...
            # relativos ao "folder_name" e o armazena na variável "datasus_path"
            datasus_path = ftp_path + folder_name + '/'
            # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
            # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
            dfi = get_dbc_info(datasus_path)
            # Adiciona um objeto pandas DataFrame como elem do objeto list "frames"
            frames.append(dfi)
        # Concatena todos os objetos pandas DataFrame que são elem de "frames" como o objeto pandas DataFrame "df_ftp"
        df_ftp = pd.concat(frames, ignore_index=True)
        # Considera apenas as linhas de "df_ftp" cuja coluna "NOME" começa pelas string "DC" ou "EE" ... ou "ST"
        df_ftp = df_ftp[df_ftp['NOME'].str.match(pat='(^DC)|(^EE)|(^EF)|(^EP)|(^EQ)|(^GM)|(^HB)|\
                                                      (^IN)|(^LT)|(^PF)|(^RC)|(^SR)|(^ST)')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" se refira aos anos de 2007
        df_ftp = df_ftp[~df_ftp['NOME'].str.match(pat='(^EE.{2}07)|(^EF.{2}07)|(^EP.{2}07)|\
                                                       (^HB.{2}07)|(^IN.{2}07)|(^RC.{2}07)')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" inicie por "GM" e se refira aos anos de 2007 a 2014
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^GM.{2}0[7-9].{2}', regex=True)]
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^GM.{2}1[0-4].{2}', regex=True)]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" se refira aos anos de 2005
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}05', regex=True)]
        # Desconsidera arquivos a partir de 2020: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}20', regex=True)]

    # SIH
    elif name_base == 'sih':
        # Diretório do host onde estão os dados da base de dados "name_base" (SIH) do Datasus
        datasus_path = '/dissemin/publicos/SIHSUS/200801_/Dados/'
        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)
        # Considera apenas as linhas de "df_ftp" cuja coluna "NOME" começa pelas string "RD" ou "SP"
        df_ftp = df_ftp[df_ftp['NOME'].str.match(pat='(^RD)|(^SP)')]
        # Desconsidera a linha de "df_ftp" cuja coluna "NOME" contenha "RDAC0909" por estar sem dado esse arquivo
        df_ftp = df_ftp[~df_ftp['NOME'].str.startswith('RDAC0909')]
        # Desconsidera arquivos a partir de 2020: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}20', regex=True)]

    # SIM
    elif name_base == 'sim':
        # Diretório do host onde estão os dados da base de dados "name_base" (SIM) do Datasus
        datasus_path = '/dissemin/publicos/SIM/CID10/DORES/'
        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)
        # Considera apenas as linhas de "df_ftp" cuja coluna "NOME" começa pela string "DO"
        df_ftp = df_ftp[df_ftp['NOME'].str.startswith('DO')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" se refira ao ano de 1996
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('1996')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" a string "BR"
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('BR')]
        # Desconsidera arquivos a partir de 2018: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^DO.{2}201[8-9]', regex=True)]

    # SINASC
    elif name_base == 'sinasc':
        # Diretório do host onde estão os dados da base de dados "name_base" (SINASC) do Datasus
        datasus_path = '/dissemin/publicos/SINASC/NOV/DNRES/'
        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)
        # Considera apenas as linhas de "df_ftp" cuja coluna "NOME" começa pela string "DN"
        df_ftp = df_ftp[df_ftp['NOME'].str.startswith('DN')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" se refira ao ano de 1996
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('1996')]
        # Desconsidera as linhas de "df_ftp" cuja coluna "NOME" a string "BR"
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('BR')]
        # Desconsidera arquivos a partir de 2018: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^DN.{2}201[8-9]', regex=True)]

    # Reset o index devido à eventual eliminação de linhas
    df_ftp.reset_index(drop=True, inplace=True)

    # Encerra conexão
    ftp.quit()

    return df_ftp


def files_in_ftp_subbase(name_subbase):
    """
    Seleciona determinadas linhas do objeto pandas DataFrame retornado pela função "get_dbc_info" e relativo
    a uma sub-base de dados do Datasus denominada "name_subbase" em lowercase.

    Parâmetros
    ----------
    name_subbase: objeto str
        String do nome (lowercase) de uma sub-base de dados do Datasus

    Retorno
    -------
    df_ftp: objeto pandas Dataframe
        Dataframe tendo como colunas o nome, o diretório e a data de inserção dos arquivos principais de dados
        selecionados de uma sub-base de dados do Datasus
    """

    # Host domain
    host = 'ftp.datasus.gov.br'
    # Criação de objeto FTP como uma variável global
    global ftp
    ftp = FTP(host)
    # Realização de conexão
    ftp.login()

    # CNES
    if name_subbase.startswith('cnes'):
        # Diretório do host onde estão os dados da sub-base de dados "name_subbase" (integrante do CNES) do Datasus
        folder = name_subbase[-2:].upper() + '/'
        datasus_path = '/dissemin/publicos/CNES/200508_/Dados/' + folder

        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)

        if name_subbase[-2:] in np.array(['ep', 'hb', 'rc', 'ee', 'ef', 'in']):
            # Desconsideração das linhas de "df_ftp" representativa de quaisquer tipos de arquivos principais de...
            # dados do cnes_ep, cnes_hb, cnes_rc, cnes_ee, cnes_ef, cnes_in do ano de 2007
            df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}07.{2}', regex=True)]
        elif name_subbase[-2:] in np.array(['gm']):
            # Desconsideração das linhas de "df_ftp" representativa de quaisquer tipos de arquivos principais de...
            # dados do cnes_gm dos anos de 2007 a 2014
            df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}0[7-9].{2}', regex=True)]
            df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}1[0-4].{2}', regex=True)]
        else:
            # Desconsideração das linhas de "ddf_ftp" representativa de quaisquer tipos de arquivos principais de...
            # dados do CNES do ano de 2005, exceto do cnes_ep, cnes_hb, cnes_rc, cnes_gm, cnes_ee, cnes_ef, cnes_in
            df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}05.{2}', regex=True)]

        # Desconsidera arquivos a partir de 2020: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}20', regex=True)]

    # SIH
    elif name_subbase.startswith('sih'):
        # Diretório do host onde estão os dados da sub-base de dados "name_subbase" (integrante do SIH) do Datasus
        datasus_path = '/dissemin/publicos/SIHSUS/200801_/Dados/'

        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)

        if name_subbase == 'sih_rd':
            # Consideração apenas das linhas do objeto pandas DataFrame "df_ftp" cuja coluna NOME inicie pela string...
            # "RD" relativa ao banco de dados das AIH Reduzidas
            df_ftp = df_ftp[df_ftp['NOME'].str.startswith('RD')]
            # Sem dados esse arquivo
            df_ftp = df_ftp[~df_ftp['NOME'].str.startswith('RDAC0909')]
        elif name_subbase == 'sih_sp':
            # Consideração apenas das linhas do objeto pandas DataFrame "df_ftp" cuja coluna NOME inicie pela string...
            # "SP" relativa ao banco de dados das AIH Reduzidas
            df_ftp = df_ftp[df_ftp['NOME'].str.startswith('SP')]

        # Desconsidera arquivos a partir de 2020: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}20', regex=True)]

    # SIA
    elif name_subbase.startswith('sia'):
        # Diretório do host onde estão os dados da sub-base de dados "name_subbase" (integrante do SIA) do Datasus
        datasus_path = '/dissemin/publicos/SIASUS/200801_/Dados/'

        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)

        if name_subbase == 'sia_pa':
            # Consideração apenas das linhas do objeto pandas DataFrame "df_ftp" cuja coluna NOME inicie pela string...
            # "PA" relativa ao banco de dados dos Procedimentos Ambulatoriais
            df_ftp = df_ftp[df_ftp['NOME'].str.startswith('PA')]
            # Adequa no nome de alguns arquivos "dbc" que terminam excepcionalmente em "A" ou "B" transformando...
            # respectivamente para "a" ou "b"
            df_ftp['NOME'].replace(regex='PASP1112A', value='PASP1112a', inplace=True)
            df_ftp['NOME'].replace(regex='PASP1112B', value='PASP1112b', inplace=True)
            df_ftp['NOME'].replace({r'^(PASP1[3-9]0[1-9])A(\.dbc)' : r'\1a\2'}, regex=True, inplace=True)
            df_ftp['NOME'].replace({r'^(PASP1[3-9]1[0-2])A(\.dbc)' : r'\1a\2'}, regex=True, inplace=True)
            df_ftp['NOME'].replace({r'^(PASP1[3-9]0[1-9])B(\.dbc)' : r'\1b\2'}, regex=True, inplace=True)
            df_ftp['NOME'].replace({r'^(PASP1[3-9]1[0-2])B(\.dbc)' : r'\1b\2'}, regex=True, inplace=True)

        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^.{4}20', regex=True)]

    # SINAN
    elif name_subbase.startswith('sinan'):
        # Diretório do host onde estão os dados da sub-base de dados "name_subbase" (integrante do SINAN) do Datasus
        datasus_path = '/dissemin/publicos/SINAN/DADOS/FINAIS/'

        # Chama a função "get_dbc_info" para colocar o nome, o diretório e a data da inserção do arquivo no...
        # endereço ftp como colunas de um objeto pandas DataFrame e os preenche com os dados de "stuff_ftp_files.txt"
        df_ftp = get_dbc_info(datasus_path)

        # Consideração apenas das linhas do objeto pandas DataFrame "df_ftp" cuja coluna NOME inicie pela string...
        # "DENG" relativa ao banco de dados dos agravos dengue e chikungunya
        df_ftp = df_ftp[df_ftp['NOME'].str.startswith('DENG')]

        # Desconsidera arquivos a partir de 2018: data wrangling ainda não realizado
        df_ftp = df_ftp[~df_ftp['NOME'].str.contains('^DENG.{2}1[8-9]', regex=True)]

    ftp.quit()

    return df_ftp


def get_tables_counts_db(pointer, name_db):
    """
    Cria dicionário dos nomes das tabelas auxiliares com o respectivo número de registros constantes de um
    banco de dados PostgreSQL.

    Parâmetros
    ----------
    pointer: objeto cursor
        Cursor de uma conexão psycopg2 com um banco de dados PostgreSQL
    name_db: objeto str
        String do nome de um banco de dados PostgreSQL

    Retorno
    -------
    dict_tables_size_pg: objeto dict
        Dicionário Python tendo como keys os nomes de tabelas auxiliares de um banco de dados PostgreSQL
        e tendo como values as respectivas quantidades de registros
    """

    if name_db == 'cnes':
        main_tables = ['dcbr', 'eebr', 'efbr', 'epbr', 'eqbr', 'gmbr', 'hbbr',
                       'inbr', 'ltbr', 'pfbr', 'rcbr', 'srbr', 'stbr']
    elif name_db == 'sih':
        main_tables = ['rdbr', 'spbr']
    elif name_db == 'sim':
        main_tables = ['dobr']
    elif name_db == 'sinasc':
        main_tables = ['dnbr']
    elif name_db == 'sinan':
        main_tables = ['dengbr']

    # Solicita a estrutura de tabelas de um banco de dados no PostgreSQL
    pointer.execute('''SELECT table_schema, table_name, ordinal_position AS position, column_name, data_type,
                              CASE WHEN character_maximum_length IS NOT NULL
                              THEN character_maximum_length
                              ELSE numeric_precision END AS max_length,
                              is_nullable,
                              column_default AS default_value
                       FROM information_schema.columns
                       WHERE table_schema NOT IN ('information_schema', 'pg_catalog')
                       ORDER BY table_schema, table_name, ordinal_position;''')
    # O banco de dados existe no PostgreSQL mas está vazio
    if pointer.rowcount == 0:
        print('\nThere is such a database in PostgreSQL but it has no tables.')
        return dict()
    # O banco de dados existe no PostgreSQL e não está vazio, mas pode estar vazio de dados
    else:
        print('\nThe database structure in PostgreSQL is shown below:\n')
        dict_tables_pg = {}
        # Iteração sobre cada linha do comando executado pelo "pointer"
        for row in pointer:
            # Impressão na tela de cada "row" do comando executado pelo "pointer"
            print('%r' % (row,))
            if row[0] == name_db:
                if row[1] not in dict_tables_pg:
                    # Preenche o objeto dict tendo como key o nome da tabela "row[1]" e...
                    # tendo como value o nome da primeira coluna ("row[3]")
                    dict_tables_pg.update({row[1]: row[3]})

        # Remove apenas as keys das tabelas principais do "name_db"
        for main_table in main_tables:
            dict_tables_pg.pop(main_table, None)
        # Cria um objeto dict para alojar como key o nome de cada tabela contida em "dict_tables_pg" e...
        # como value a quantidade de registros da tabela
        dict_tables_size_pg = {}
        # Iteração sobre cada nome de tabela contido em "dict_tables_pg"
        for table in dict_tables_pg:
            # Obtém a contagem de registros da tabela "table" do "name_db"
            pointer.execute(f'''SELECT COUNT('{dict_tables_pg[table]}') FROM {name_db}.{table}''')
            for row in pointer:
                # Coleta o número de registros contidos em "table"
                size = row[0]
                # Preenche um objeto dict tendo como key o nome da tabela "table" e...
                # como value o número de registros "size"
                dict_tables_size_pg.update({table: size})
        return dict_tables_size_pg


def get_tables_counts_subdb(pointer, name_subdb):
    """
    Cria dicionário dos nomes das tabelas auxiliares com o respectivo número de registros constantes de um
    sub-banco de dados PostgreSQL.

    Parâmetros
    ----------
    pointer: objeto cursor
        Cursor de uma conexão psycopg2 com um sub-banco de dados PostgreSQL
    name_subdb: objeto str
        String do nome de um sub-banco de dados PostgreSQL

    Retorno
    -------
    dict_tables_size_pg: objeto dict
        Dicionário Python tendo como keys os nomes de tabelas auxiliares de um sub-banco de dados PostgreSQL
        e tendo como values as respectivas quantidades de registros
    """

    if name_subdb.startswith('cnes') or name_subdb.startswith('sih') or name_subdb.startswith('sia'):
        main_table = name_subdb[-2:] + 'br'
    elif name_subdb == 'sim':
        main_table = 'dobr'
    elif name_subdb == 'sinasc':
        main_table = 'dnbr'
    elif name_subdb == 'sinan_deng':
        main_table = 'dengbr'

    # Solicita a estrutura de tabelas de um banco de dados no PostgreSQL
    pointer.execute('''SELECT table_schema, table_name, ordinal_position AS position, column_name, data_type,
                              CASE WHEN character_maximum_length IS NOT NULL
                              THEN character_maximum_length
                              ELSE numeric_precision END AS max_length,
                              is_nullable,
                              column_default AS default_value
                       FROM information_schema.columns
                       WHERE table_schema NOT IN ('information_schema', 'pg_catalog')
                       ORDER BY table_schema, table_name, ordinal_position;''')
    # O banco de dados existe no PostgreSQL mas está vazio
    if pointer.rowcount == 0:
        print('\nThere is such a database in PostgreSQL but it has no tables.')
        return dict()
    # O banco de dados existe no PostgreSQL e não está vazio
    else:
        print('\nThe database structure in PostgreSQL is shown below:\n')
        dict_tables_pg = {}
        # Iteração sobre cada linha do comando executado pelo "pointer"
        for row in pointer:
            # Impressão na tela de cada "row" do comando executado pelo "pointer"
            print('%r' % (row,))
            if row[0] == name_subdb:
                if row[1] not in dict_tables_pg:
                    # Preenche o objeto dict tendo como key o nome da tabela "row[1]" e como value o nome da...
                    # primeira coluna ("row[3]")
                    dict_tables_pg.update({row[1]: row[3]})

        # Remove apenas a key da tabela principal do "name_subdb"
        dict_tables_pg.pop(main_table, None)
        # Cria um objeto dict para alojar como key o nome de cada tabela contida em "dict_tables_pg" e como...
        # value a quantidade de registros da tabela
        dict_tables_size_pg = {}
        # Iteração sobre cada nome de tabela contido em "dict_tables_pg"
        for table in dict_tables_pg:
            # Obtém a contagem de registros da tabela "table" do "name_subdb"
            pointer.execute(f'''SELECT COUNT('{dict_tables_pg[table]}') FROM {name_subdb}.{table}''')
            for row in pointer:
                # Coleta o número de registros contidos em "table"
                size = row[0]
                # Preenche um objeto dict tendo como key o nome da tabela "table" e como value o número de...
                # registros "size"
                dict_tables_size_pg.update({table: size})
        return dict_tables_size_pg


def files_loaded(structure_pg, name_db, e):
    """
    Obtém lista de nomes (e outros metadados) dos arquivos principais de dados eventualmente carregados num
    banco ou sub-banco de dados PostgreSQL.

    Parâmetros
    ----------
    structure_pg: objeto dict
        Dicionário Python tendo como keys os nomes de tabelas auxiliares de um banco ou sub-banco de dados
        PostgreSQL e tendo como values as respectivas quantidades de registros
    name_db: objeto str
        String do nome de um banco ou sub-banco de dados PostgreSQL
    e: objeto engine
        Engine de comunicação com um banco ou sub-banco de dados PostgreSQL através do driver psycopg2

    Retorno
    -------
    df_files_pg: objeto pandas DataFrame
        Dataframe contendo nomes (e outros metadados) dos arquivos principais de dados eventualmente carregados
        num banco ou sub-banco de dados PostgreSQL
    """

    # O schema do "name_db" não foi inserido no PostgreSQL
    if structure_pg == dict():
        print('\nThe schema of the PostgreSQL database has no tables at all!')
        # Cria um objeto pandas DataFrame vazio com as colunas especificadas
        df_files_pg = pd.DataFrame(columns=['NOME', 'DIRETORIO', 'DATA_INSERCAO_FTP', 'DATA_HORA_CARGA', 'QTD_REGISTROS'])
        # Coloca a coluna NOME como index do objeto pandas DataFrame
        df_files_pg.set_index('NOME')
        return df_files_pg
    # Tabela "arquivos" não constante ainda do "name_db" do PostgreSQL
    elif 'arquivos' not in structure_pg:
        print('\nThe schema of the PostgreSQL database does not contain the table arquivos.')
        # Cria um objeto pandas DataFrame vazio com as colunas especificadas
        df_files_pg = pd.DataFrame(columns=['NOME', 'DIRETORIO', 'DATA_INSERCAO_FTP', 'DATA_HORA_CARGA', 'QTD_REGISTROS'])
        # Coloca a coluna NOME como index do objeto pandas DataFrame
        df_files_pg.set_index('NOME')
        return df_files_pg
    # Tabela "arquivos" já constante do "name_db" do PostgreSQL
    elif 'arquivos' in structure_pg:
        print('\nThe schema of the PostgreSQL database contains the table arquivos.')
        # Coleta os nomes dos arquivos de dados da "name_db" constantes do PostgreSQL e suas informações...
        # como um objeto pandas DataFrame
        df_files_pg = pd.read_sql(f'''SELECT * FROM {name_db}.arquivos''', con=e, index_col='NOME')
        if df_files_pg.shape[0] == 0:
            print('\nThe PostgreSQL database doest not contain main data file loaded.')
        else:
            print('\nThe PostgreSQL database contains main data file loaded.')
        return df_files_pg


def files_to_load(df_files_ftp, df_files_pg):
    """
    Obtém lista de nomes (e outros metadados) dos arquivos principais de dados que faltam carregar num
    banco ou sub-banco de dados PostgreSQL.

    Parâmetros
    ----------
    df_files_ftp: objeto pandas DataFrame
        Dataframe contendo nomes (e outros metadados) dos arquivos principais de dados selecionados de uma base
        de dados do Datasus
    df_files_pg: objeto pandas DataFrame
        Dataframe contendo nomes (e outros metadados) dos arquivos principais de dados eventualmente carregados
        num banco ou sub-banco de dados PostgreSQL

    Retorno
    -------
    df_difference: objeto pandas DataFrame
        Dataframe contendo nomes (e outros metadados) dos arquivos principais de dados que faltam carregar num
        banco ou sub-banco de dados PostgreSQL
    """

    # Objeto list de nomes de arquivos de dados "dbc" do banco de dados "datasus_db" do Datasus contidos no...
    # diretório do seu endereço ftp referenciado à variável "datasus_path" e não presentes no banco...
    # de dados "DB_NAME" do PostgreSQL
    list_files_exclusivos_ftp = list(set(df_files_ftp.NOME.tolist()) - set(df_files_pg.index.tolist()))
    # Criação de objeto pandas DataFrame para conter a listagem de nomes de arquivos de dados "dbc" que faltam ser...
    # carregados contendo ainda o diretório e a data de inserção de cada arquivo de dados "dbc"
    df_difference = pd.DataFrame(columns=['NOME', 'DIRETORIO', 'DATA_INSERCAO_FTP'])
    # Iteração sobre cada nome de arquivo "dbc" que falta ser carregado
    for file_name in list_files_exclusivos_ftp:
        # Coleta a linha do objeto pandas DataFrame "df_files_ftp" que contém na coluna NOME o valor "file_name"
        row = df_files_ftp.loc[df_files_ftp['NOME'] == file_name]
        # Aloca a linha "row" no objeto pandas DataFrame "df_difference"
        df_difference = df_difference.append(row, ignore_index=True)
    # Ordena as linhas de "df_difference" por ordem crescente dos valores das colunas NOME e DIRETORIO
    df_difference.sort_values(['NOME', 'DIRETORIO'], inplace=True)
    # Reset o index devido ao sorting prévio
    df_difference.reset_index(drop=True, inplace=True)
    return df_difference



###########################################################################################################################################################################
# SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA SIA_PA #
###########################################################################################################################################################################

import psycopg2


"""
Cria o schema do banco de dados do SIA_PA (Procedimentos Ambulatoriais) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e SQL Server com no máximo pequenas modificações.

"""


# Função de criação das tabelas do banco de dados sia_pa
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela Procedimentos Ambulatoriais (PA) (tabela principal do "sub" banco de dados SIA_PA)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pabr("UF_PA"                  VARCHAR(2),
                                                                  "ANO_PA"                 INTEGER,
                                                                  "MES_PA"                 VARCHAR(2),
                                                                  "PACODUNI_ID"            VARCHAR(7),
                                                                  "PAGESTAO_ID"            VARCHAR(6),
                                                                  "PACONDIC_ID"            VARCHAR(2),
                                                                  "PAUFMUN_ID"             VARCHAR(6),
                                                                  "PAREGCT_ID"             VARCHAR(4),
                                                                  "PA_INCOUT"              NUMERIC,
                                                                  "PA_INCURG"              NUMERIC,
                                                                  "PATPUPS_ID"             VARCHAR(2),
                                                                  "PATIPPRE_ID"            VARCHAR(2),
                                                                  "PA_MN_IND"              VARCHAR(2),
                                                                  "PA_CNPJCPF"             VARCHAR(14),
                                                                  "PA_CNPJMNT"             VARCHAR(14),
                                                                  "PA_CNPJ_CC"             VARCHAR(14),
                                                                  "PA_MVM"                 DATE,
                                                                  "PA_CMP"                 DATE,
                                                                  "PAPROC_ID"              VARCHAR(10),
                                                                  "PATPFIN_ID"             VARCHAR(2),
                                                                  "PANIVCPL_ID"            VARCHAR(2),
                                                                  "PADOCORIG_ID"           VARCHAR(2),
                                                                  "PA_AUTORIZ"             VARCHAR(20),
                                                                  "PA_CNSMED"              VARCHAR(15),
                                                                  "PACBOCOD_ID"            VARCHAR(6),
                                                                  "PAMOTSAI_ID"            VARCHAR(2),
                                                                  "PA_OBITO"               NUMERIC,
                                                                  "PA_ENCERR"              NUMERIC,
                                                                  "PA_PERMAN"              NUMERIC,
                                                                  "PA_ALTA"                NUMERIC,
                                                                  "PA_TRANSF"              NUMERIC,
                                                                  "PACIDPRI_ID"            VARCHAR(4),
                                                                  "PACIDSEC_ID"            VARCHAR(4),
                                                                  "PACIDCAS_ID"            VARCHAR(4),
                                                                  "PACATEND_ID"            VARCHAR(2),
                                                                  "PA_IDADE"               FLOAT,
                                                                  "IDADEMIN"               FLOAT,
                                                                  "IDADEMAX"               FLOAT,
                                                                  "PAFLIDADE_ID"           VARCHAR(2),
                                                                  "PASEXO_ID"              VARCHAR(2),
                                                                  "PARACACOR_ID"           VARCHAR(2),
                                                                  "PAMUNPCN_ID"            VARCHAR(6),
                                                                  "PA_QTDPRO"              FLOAT,
                                                                  "PA_QTDAPR"              FLOAT,
                                                                  "PA_VALPRO"              FLOAT,
                                                                  "PA_VALAPR"              FLOAT,
                                                                  "PA_UFDIF"               NUMERIC,
                                                                  "PA_MNDIF"               NUMERIC,
                                                                  "PA_DIF_VAL"             FLOAT,
                                                                  "NU_VPA_TOT"             FLOAT,
                                                                  "NU_PA_TOT"              FLOAT,
                                                                  "PAINDICA_ID"            VARCHAR(2),
                                                                  "PACODOCO_ID"            VARCHAR(2),
                                                                  "PA_FLER"                NUMERIC,
                                                                  "PAETNIA_ID"             VARCHAR(4),
                                                                  "PA_VL_CF"               FLOAT,
                                                                  "PA_VL_CL"               FLOAT,
                                                                  "PA_VL_INC"              FLOAT,
                                                                  "PASRCC_ID"              VARCHAR(6),
                                                                  "PAINE_ID"               VARCHAR(10),
                                                                  "PANATJUR_ID"            VARCHAR(4),
                                                                  "GRUPO_ID"               VARCHAR(2),
                                                                  "SUBGRUPO_ID"            VARCHAR(4),
                                                                  "FORMA_ID"               VARCHAR(6));

                       /*Tabela dos estabelecimentos de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacoduni("ID"                 VARCHAR(7),
                                                                      "DESCESTAB"          VARCHAR(66),
                                                                      "RSOC_MAN"           VARCHAR(66),
                                                                      "CPF_CNPJ"           VARCHAR(14),
                                                                      "EXCLUIDO"           NUMERIC,
                                                                      "DATAINCL"           DATE,
                                                                      "DATAEXCL"           DATE);

                       /*Tabela dos municípios de gestão*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pagestao("ID"                 VARCHAR(6),
                                                                      "MUNNOME"	           VARCHAR(66),
                                                                      "MUNNOMEX"           VARCHAR(66),
                                                                      "MUNCODDV"           VARCHAR(7),
                                                                      "OBSERV"             VARCHAR(66),
                                                                      "SITUACAO"           VARCHAR(66),
                                                                      "MUNSINP"	           VARCHAR(66),
                                                                      "MUNSIAFI"           VARCHAR(66),
                                                                      "UFCOD_ID"           VARCHAR(2),
                                                                      "AMAZONIA"           VARCHAR(66),
                                                                      "FRONTEIRA"          VARCHAR(66),
                                                                      "CAPITAL"            VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"           FLOAT,
                                                                      "LONGITUDE"          FLOAT,
                                                                      "ALTITUDE"           FLOAT,
                                                                      "AREA"               FLOAT,
                                                                      "ANOINST"            VARCHAR(66),
                                                                      "ANOEXT"             VARCHAR(66),
                                                                      "SUCESSOR"           VARCHAR(66));

                       /*Tabela dos tipos de gestão*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacondic("ID"                 VARCHAR(2),
                                                                      "GESTAO"             VARCHAR(66));

                       /*Tabela dos municípios de localização do estabelecimento de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paufmun("ID"                  VARCHAR(6),
                                                                     "MUNNOME"	           VARCHAR(66),
                                                                     "MUNNOMEX"            VARCHAR(66),
                                                                     "MUNCODDV"            VARCHAR(7),
                                                                     "OBSERV"              VARCHAR(66),
                                                                     "SITUACAO"            VARCHAR(66),
                                                                     "MUNSINP"	           VARCHAR(66),
                                                                     "MUNSIAFI"            VARCHAR(66),
                                                                     "UFCOD_ID"            VARCHAR(2),
                                                                     "AMAZONIA"            VARCHAR(66),
                                                                     "FRONTEIRA"           VARCHAR(66),
                                                                     "CAPITAL"             VARCHAR(66),
                                                                     "RSAUDE_ID"           VARCHAR(5),
                                                                     "LATITUDE"            FLOAT,
                                                                     "LONGITUDE"           FLOAT,
                                                                     "ALTITUDE"            FLOAT,
                                                                     "AREA"                FLOAT,
                                                                     "ANOINST"             VARCHAR(66),
                                                                     "ANOEXT"              VARCHAR(66),
                                                                     "SUCESSOR"            VARCHAR(66));

                       /*Tabela dos tipos de regra contratual*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paregct("ID"                  VARCHAR(4),
                                                                     "REGRA"               VARCHAR(66));

                       /*Tabela dos tipos de estabelecimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.patpups("ID"                  VARCHAR(2),
                                                                     "ESTABELECIMENTO"     VARCHAR(66));

                       /*Tabela dos tipos de prestador*/
                       CREATE TABLE IF NOT EXISTS {child_db}.patippre("ID"                 VARCHAR(2),
                                                                      "ESFERA"             VARCHAR(66));

                       /*Tabela dos tipos de procedimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paproc("ID"                   VARCHAR(10),
                                                                    "PROCEDIMENTO"         VARCHAR(100));

                       /*Tabela dos tipos de financiamento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.patpfin("ID"                  VARCHAR(2),
                                                                     "FINANCIAMENTO"       VARCHAR(66));

                       /*Tabela dos níveis de complexidade*/
                       CREATE TABLE IF NOT EXISTS {child_db}.panivcpl("ID"                 VARCHAR(2),
                                                                      "COMPLEXIDADE"       VARCHAR(66));

                       /*Tabela dos tipos de sub-bancos de dados do SIA*/
                       CREATE TABLE IF NOT EXISTS {child_db}.padocorig("ID"                VARCHAR(2),
                                                                       "TIPO_SIA"          VARCHAR(66));

                       /*Tabela das ocupações*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacbocod("ID"                 VARCHAR(6),
                                                                      "OCUPACAO"           VARCHAR(66));

                       /*Tabela dos motivos de saída*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pamotsai("ID"                 VARCHAR(2),
                                                                      "MOTIVO"             VARCHAR(66));

                       /*Tabela dos diagnósticos primários*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacidpri("ID"                 VARCHAR(4),
                                                                      "DIAGNOSTICO"        VARCHAR(66));

                       /*Tabela dos diagnósticos secundários*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacidsec("ID"                 VARCHAR(4),
                                                                      "DIAGNOSTICO"        VARCHAR(66));

                       /*Tabela dos diagnósticos associados*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacidcas("ID"                 VARCHAR(4),
                                                                      "DIAGNOSTICO"        VARCHAR(66));

                       /*Tabela dos tipos de caráter do atendimento*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacatend("ID"                 VARCHAR(2),
                                                                      "CARATER"            VARCHAR(66));

                       /*Tabela dos tipos de compatibilidade de idade*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paflidade("ID"                VARCHAR(2),
                                                                       "COMPATIBILIDADE"   VARCHAR(66));

                       /*Tabela dos tipos de sexo*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pasexo("ID"                   VARCHAR(2),
                                                                    "SEXO"                 VARCHAR(66));

                       /*Tabela dos tipos de raça*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paracacor("ID"                VARCHAR(2),
                                                                       "RACA"              VARCHAR(66));

                       /*Tabela dos municípios de residência do paciente*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pamunpcn("ID"                 VARCHAR(6),
                                                                      "MUNNOME"	           VARCHAR(66),
                                                                      "MUNNOMEX"           VARCHAR(66),
                                                                      "MUNCODDV"           VARCHAR(7),
                                                                      "OBSERV"             VARCHAR(66),
                                                                      "SITUACAO"           VARCHAR(66),
                                                                      "MUNSINP"	           VARCHAR(66),
                                                                      "MUNSIAFI"           VARCHAR(66),
                                                                      "UFCOD_ID"           VARCHAR(2),
                                                                      "AMAZONIA"           VARCHAR(66),
                                                                      "FRONTEIRA"          VARCHAR(66),
                                                                      "CAPITAL"            VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"           FLOAT,
                                                                      "LONGITUDE"          FLOAT,
                                                                      "ALTITUDE"           FLOAT,
                                                                      "AREA"               FLOAT,
                                                                      "ANOINST"            VARCHAR(66),
                                                                      "ANOEXT"             VARCHAR(66),
                                                                      "SUCESSOR"           VARCHAR(66));

                       /*Tabela dos tipos de situação da produção*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paindica("ID"                 VARCHAR(2),
                                                                      "SITUACAO"           VARCHAR(66));

                       /*Tabela dos tipos de ocorrência da produção*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pacodoco("ID"                 VARCHAR(2),
                                                                      "OCORRENCIA"         VARCHAR(66));

                       /*Tabela dos tipos de etnia indígena*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paetnia("ID"                  VARCHAR(4),
                                                                     "INDIGENA"            VARCHAR(66));

                       /*Tabela das ocupações*/
                       CREATE TABLE IF NOT EXISTS {child_db}.pasrcc("ID"                   VARCHAR(6),
                                                                     "OCUPACAO"            VARCHAR(66));

                       /*Tabela dos tipos de equipes*/
                       CREATE TABLE IF NOT EXISTS {child_db}.paine("ID"                    VARCHAR(10),
                                                                   "EQUIPE"                VARCHAR(66));

                       /*Tabela dos tipos de equipes*/
                       CREATE TABLE IF NOT EXISTS {child_db}.panatjur("ID"                 VARCHAR(4),
                                                                      "NATUREZA"           VARCHAR(66));

                      /*Tabela dos Estados da RFB*/
                      CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                  "ESTADO"                 VARCHAR(66),
                                                                  "SIGLA_UF"               VARCHAR(66));

                      /*Tabela de regiões de saúde IBGE*/
                      CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                   "REGIAO"                VARCHAR(66));

                      /*Tabela dos grupos de procedimentos*/
                      CREATE TABLE IF NOT EXISTS {child_db}.grupo("ID"                     VARCHAR(2),
                                                                  "GRUPO"                  VARCHAR(50));

                      /*Tabela dos subgrupos de procedimentos*/
                      CREATE TABLE IF NOT EXISTS {child_db}.subgrupo("ID"                  VARCHAR(4),
                                                                     "SUBGRUPO"            VARCHAR(100));

                     /*Tabela das formas de procedimentos*/
                     CREATE TABLE IF NOT EXISTS {child_db}.forma("ID"                      VARCHAR(6),
                                                                 "FORMA"                   VARCHAR(100));

                     /*Tabela de Arquivos*/
                     CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                 VARCHAR(15),
                                                                    "DIRETORIO"            VARCHAR(66),
                                                                    "DATA_INSERCAO_FTP"    DATE,
                                                                    "DATA_HORA_CARGA"      TIMESTAMP,
                                                                    "QTD_REGISTROS"        INTEGER);
                    ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()



###########################################################################################################################################################
# CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC CNES_DC #
###########################################################################################################################################################

import psycopg2


"""
Cria o schema do sub banco de dados do CNES_DC (Dados Complementares) para atender ao formato do SGBD postgreSQL.
Na verdade o schema abaixo deve atender ao formato dos SGBD Oracle e SQL Server com no máximo pequenas modificações.

"""

# Função de criação das tabelas do sub banco de dados cnes_dc
def create_tables(connection_data, child_db):
    conn = psycopg2.connect(dbname=connection_data[0],
                            host=connection_data[1],
                            port=connection_data[2],
                            user=connection_data[3],
                            password=connection_data[4])

    cursor = conn.cursor()

    cursor.execute(f'''/*Tabela Dados Complementares (tabela principal do sub banco de dados cnes_dc)*/
                       CREATE TABLE IF NOT EXISTS {child_db}.dcbr("CNES_ID"                 VARCHAR(7),
                                                                  "UF_DC"                   VARCHAR(2),
                                                                  "ANO_DC"                  INTEGER,
                                                                  "MES_DC"                  VARCHAR(2),
                                                                  "CODUFMUN_ID"             VARCHAR(6),
                                                                  "S_HBSAGP"                FLOAT,
                                                                  "S_HBSAGN"                FLOAT,
                                                                  "S_DPI"                   FLOAT,
                                                                  "S_DPAC"                  FLOAT,
                                                                  "S_REAGP"                 FLOAT,
                                                                  "S_REAGN"                 FLOAT,
                                                                  "S_REHCV"                 FLOAT,
                                                                  "MAQ_PROP"                FLOAT,
                                                                  "MAQ_OUTR"                FLOAT,
                                                                  "F_AREIA"                 NUMERIC,
                                                                  "F_CARVAO"                NUMERIC,
                                                                  "ABRANDAD"                NUMERIC,
                                                                  "DEIONIZA"                NUMERIC,
                                                                  "OSMOSE_R"                NUMERIC,
                                                                  "OUT_TRAT"                NUMERIC,
                                                                  "CNS_NEFR"                VARCHAR(15),
                                                                  "DIALISE"                 NUMERIC,
                                                                  "SIMUL_RD"                FLOAT,
                                                                  "PLANJ_RD"                FLOAT,
                                                                  "ARMAZ_FT"                FLOAT,
                                                                  "CONF_MAS"                FLOAT,
                                                                  "SALA_MOL"                FLOAT,
                                                                  "BLOCOPER"                FLOAT,
                                                                  "S_ARMAZE"                FLOAT,
                                                                  "S_PREPAR"                FLOAT,
                                                                  "S_QCDURA"                FLOAT,
                                                                  "S_QLDURA"                FLOAT,
                                                                  "S_CPFLUX"                FLOAT,
                                                                  "S_SIMULA"                FLOAT,
                                                                  "S_ACELL6"                FLOAT,
                                                                  "S_ALSEME"                FLOAT,
                                                                  "S_ALCOME"                FLOAT,
                                                                  "ORTV1050"                FLOAT,
                                                                  "ORV50150"                FLOAT,
                                                                  "OV150500"                FLOAT,
                                                                  "UN_COBAL"                FLOAT,
                                                                  "EQBRBAIX"                FLOAT,
                                                                  "EQBRMEDI"                FLOAT,
                                                                  "EQBRALTA"                FLOAT,
                                                                  "EQ_MAREA"                FLOAT,
                                                                  "EQ_MINDI"                FLOAT,
                                                                  "EQSISPLN"                FLOAT,
                                                                  "EQDOSCLI"                FLOAT,
                                                                  "EQFONSEL"                FLOAT,
                                                                  "CNS_ADM"                 VARCHAR(15),
                                                                  "CNS_OPED"                VARCHAR(15),
                                                                  "CNS_CONC"                VARCHAR(15),
                                                                  "CNS_OCLIN"               VARCHAR(15),
                                                                  "CNS_MRAD"                VARCHAR(15),
                                                                  "CNS_FNUC"                VARCHAR(15),
                                                                  "QUIMRADI"                NUMERIC,
                                                                  "S_RECEPC"                FLOAT,
                                                                  "S_TRIHMT"                FLOAT,
                                                                  "S_TRICLI"                FLOAT,
                                                                  "S_COLETA"                FLOAT,
                                                                  "S_AFERES"                FLOAT,
                                                                  "S_PREEST"                FLOAT,
                                                                  "S_PROCES"                FLOAT,
                                                                  "S_ESTOQU"                FLOAT,
                                                                  "S_DISTRI"                FLOAT,
                                                                  "S_SOROLO"                FLOAT,
                                                                  "S_IMUNOH"                FLOAT,
                                                                  "S_PRETRA"                FLOAT,
                                                                  "S_HEMOST"                FLOAT,
                                                                  "S_CONTRQ"                FLOAT,
                                                                  "S_BIOMOL"                FLOAT,
                                                                  "S_IMUNFE"                FLOAT,
                                                                  "S_TRANSF"                FLOAT,
                                                                  "S_SGDOAD"                FLOAT,
                                                                  "QT_CADRE"                FLOAT,
                                                                  "QT_CENRE"                FLOAT,
                                                                  "QT_REFSA"                FLOAT,
                                                                  "QT_CONRA"                FLOAT,
                                                                  "QT_EXTPL"                FLOAT,
                                                                  "QT_FRE18"                FLOAT,
                                                                  "QT_FRE30"                FLOAT,
                                                                  "QT_AGIPL"                FLOAT,
                                                                  "QT_SELAD"                FLOAT,
                                                                  "QT_IRRHE"                FLOAT,
                                                                  "QT_AGLTN"                FLOAT,
                                                                  "QT_MAQAF"                FLOAT,
                                                                  "QT_REFRE"                FLOAT,
                                                                  "QT_REFAS"                FLOAT,
                                                                  "QT_CAPFL"                FLOAT,
                                                                  "CNS_HMTR"                VARCHAR(15),
                                                                  "CNS_HMTL"                VARCHAR(15),
                                                                  "CNS_CRES"                VARCHAR(15),
                                                                  "CNS_RTEC"                VARCHAR(15),
                                                                  "HEMOTERA"                NUMERIC);

                       /*Tabela dos estabelecimentos de saúde*/
                       CREATE TABLE IF NOT EXISTS {child_db}.cnes("ID"                      VARCHAR(7),
                                                                  "DESCESTAB"               VARCHAR(66),
                                                                  "RSOC_MAN"                VARCHAR(66),
                                                                  "CPF_CNPJ"                VARCHAR(14),
                                                                  "EXCLUIDO"                NUMERIC,
                                                                  "DATAINCL"                DATE,
                                                                  "DATAEXCL"                DATE);

                       /*Tabela dos municípios de localização de estabelecimentos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.codufmun("ID"                  VARCHAR(6),
                                                                      "MUNNOME"	            VARCHAR(66),
                                                                      "MUNNOMEX"            VARCHAR(66),
                                                                      "MUNCODDV"            VARCHAR(7),
                                                                      "OBSERV"              VARCHAR(66),
                                                                      "SITUACAO"            VARCHAR(66),
                                                                      "MUNSINP"	            VARCHAR(66),
                                                                      "MUNSIAFI"            VARCHAR(66),
                                                                      "UFCOD_ID"            VARCHAR(2),
                                                                      "AMAZONIA"            VARCHAR(66),
                                                                      "FRONTEIRA"           VARCHAR(66),
                                                                      "CAPITAL"             VARCHAR(66),
                                                                      "RSAUDE_ID"           VARCHAR(5),
                                                                      "LATITUDE"            FLOAT,
                                                                      "LONGITUDE"           FLOAT,
                                                                      "ALTITUDE"            FLOAT,
                                                                      "AREA"                FLOAT,
                                                                      "ANOINST"             VARCHAR(66),
                                                                      "ANOEXT"              VARCHAR(66),
                                                                      "SUCESSOR"            VARCHAR(66));

                       /*Tabela dos Estados da RFB*/
                       CREATE TABLE IF NOT EXISTS {child_db}.ufcod("ID"                     VARCHAR(2),
                                                                   "ESTADO"                 VARCHAR(66),
                                                                   "SIGLA_UF"               VARCHAR(66));

                       /*Tabela de regiões de saúde IBGE*/
                       CREATE TABLE IF NOT EXISTS {child_db}.rsaude("ID"                    VARCHAR(5),
                                                                    "REGIAO"                VARCHAR(66));

                       /*Tabela de Arquivos*/
                       CREATE TABLE IF NOT EXISTS {child_db}.arquivos("NOME"                VARCHAR(15),
                                                                      "DIRETORIO"           VARCHAR(66),
                                                                      "DATA_INSERCAO_FTP"   DATE,
                                                                      "DATA_HORA_CARGA"     TIMESTAMP,
                                                                      "QTD_REGISTROS"       INTEGER);
                ''')
    conn.commit()
    # Encerra o cursor
    cursor.close()
    # Encerra a conexão
    conn.close()
